# create ragu config file
import configparser

def create_config(fpath):
    
    config = configparser.ConfigParser(allow_no_value=True, delimiters='=')
    config.add_section('param')
    config.set('param', '# str uid: user id')
    config.set('param', 'uid', '')
    config.set('param', '# str cmap: Matplotlib colormap to use (default = seismic)')
    config.set('param', 'cmap', '')

    config.add_section('path')
    config.set('path', '# str datPath: path to data files (optional)')
    config.set('path', 'datPath', '')
    config.set('path', '# str simPath: path to clutter simulations (optional)')
    config.set('path', 'simPath', '')
    config.set('path', '# str mapPath: path to basemap geotiff files (optional)')
    config.set('path', 'mapPath', '')
    config.set('path', '# str outPath: output path (optional)')
    config.set('path', 'outPath', '')

    config.add_section('nav')
    config.set('nav', '# str body: planetary body from which radar data was acquired (earth, moon, mars)')
    config.set('nav', 'body', 'earth')
    config.set('nav', '# str navcrs: crs string')
    config.set('nav', '# wgs84 projection string for earth: +proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs')
    config.set('nav', '# longlat projection string for mars: +proj=longlat +a=3396190 +b=3376200 +no_defs')
    config.set('nav', 'crs', '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs')

    config.add_section('output')
    config.set('output', '# float asep: antenna separation in meters (optional)')
    config.set('output', 'asep', '')
    config.set('output', '# float eps_r: relative permittivity (dielectric constant), required for plotting in depth and calculating layer thickness')
    config.set('output', 'eps_r', '3.15')
    config.set('output','# bool amp: export pick amplitudes')
    config.set('output','amp', 'True')
    config.set('output','# bool csv: export csv file of picks')
    config.set('output','csv', 'True')
    config.set('output','# bool gpkg: export geopackage of picks')
    config.set('output','gpkg', 'True')
    config.set('output','# bool fig: export radargram figure with any existing picks')
    config.set('output','fig', 'True')
    with open(fpath, 'w') as f:
        config.write(f)


# RAGU - Radar Analysis Graphical Utility
#
# copyright © 2020 btobers <tobers.brandon@gmail.com>
#
# distributed under terms of the GNU GPL3.0 license
"""
rdata flags to hold the key information for a radar profile
"""

class flags(object):
    def __init__(self):
        # basic data file attributes
        #: sampzero, zero sample data setting following time zero adjustment, or radar data flattening
        self.sampzero = 0
        #: sim, bool clutter simulation present
        self.sim = False


# RAGU - Radar Analysis Graphical Utility
#
# copyright © 2020 btobers <tobers.brandon@gmail.com>
#
# distributed under terms of the GNU GPL3.0 license
"""
basemap class is a tkinter frame which handles the RAGU basemap
"""
### imports ###
from ragu.nav import navparse
from ragu.tools import utils
import numpy as np
import tkinter as tk
import rasterio as rio
import os, glob
from pyproj import Transformer
import matplotlib as mpl
mpl.use("TkAgg")
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
# try:
#     plt.rcParams["font.family"] = "Times New Roman"
# except:
#     pass

class basemap(tk.Frame):
    def __init__(self, parent, datPath, navcrs, body, to_gui):
        self.parent = parent
        self.datPath = datPath
        self.navcrs = navcrs
        self.body = body
        self.to_gui = to_gui
        # create tkinter toplevel window to display basemap
        self.basemap_window = tk.Toplevel(self.parent)
        self.ontop_bool = tk.BooleanVar(value=0)
        self.basemap_window.wm_attributes("-topmost", "false")
        img = tk.PhotoImage(file=os.path.join(os.path.dirname(__file__), "../recs", "basemap_icon.png"))
        self.basemap_window.tk.call("wm", "iconphoto", self.basemap_window._w, img)
        self.basemap_window.config(bg="#d9d9d9")
        self.basemap_window.title("RAGU - Map Window")
        self.map_display = tk.Frame(self.basemap_window)
        self.map_display.pack(side="bottom", fill="both", expand=1)
        # bind ctrl-q key to basemap_close()
        self.basemap_window.bind("<Control-q>", self.basemap_close)
        # bind x-out to basemap_close()
        self.basemap_window.protocol("WM_DELETE_WINDOW", self.basemap_close)
        self.cmap = tk.StringVar(value="Greys_r")
        self.setup()


    # setup the tkinter window
    def setup(self):
        # show basemap figure in basemap window
        # generate menubar
        menubar = tk.Menu(self.basemap_window)
        fileMenu = tk.Menu(menubar, tearoff=0)
        windowMenu = tk.Menu(menubar, tearoff=0)

        # settings submenu
        loadMenu = tk.Menu(fileMenu,tearoff=0)
        loadMenu.add_command(label="select files", command=self.load_tracks)
        loadMenu.add_command(label="select folder", command= lambda: self.load_tracks(dir = True))
        fileMenu.add_cascade(label="load tracks", menu = loadMenu)

        fileMenu.add_command(label="clear tracks", command=self.clear_nav)
        fileMenu.add_command(label="preferences", command=self.settings)
        fileMenu.add_command(label="exit       [ctrl+q]", command=self.basemap_close)

        # windowMenu.add(label="Stay on top", menu = windowMenu, command=self.ontop)
        windowMenu.add_checkbutton(label="Stay on top", onvalue=1, offvalue=0, variable=self.ontop_bool, command=self.ontop)

        # # add items to menubar
        menubar.add_cascade(label="file", menu=fileMenu)
        menubar.add_cascade(label="window", menu=windowMenu)
        
        # add the menubar to the window
        self.basemap_window.config(menu=menubar)

        # set up info frame
        infoFrame = tk.Frame(self.basemap_window)
        infoFrame.pack(side="top",fill="both")
        # button to toggle track visibility
        self.track_viz = tk.BooleanVar(value=True)
        tk.Label(infoFrame, text="track display: ").pack(side="left")
        tk.Radiobutton(infoFrame,text="all", variable=self.track_viz, value=True, command=self.plot_tracks).pack(side="left")
        tk.Radiobutton(infoFrame,text="current", variable=self.track_viz, value=False, command=self.plot_tracks).pack(side="left")

        # initialize the basemap figure
        self.map_fig = mpl.figure.Figure()
        self.map_fig.patch.set_facecolor(self.parent.cget("bg"))
        self.map_fig_ax = self.map_fig.add_subplot(111)
        self.bm_im = self.map_fig_ax.imshow(np.ones((100,100)), cmap=self.cmap.get(), aspect="auto")
        self.map_fig_ax.set_visible(False)
        self.map_fig_ax.set(xlabel = "x [km]", ylabel = "y [km]")
        # initialize artists
        self.track_ln, = self.map_fig_ax.plot([], [], "k.", ms=.1, picker=True)
        self.track_start_ln, = self.map_fig_ax.plot([], [], "go", ms=3, label="start")
        self.track_end_ln, = self.map_fig_ax.plot([], [], "ro", ms=3, label="end")
        # pack mpl figure in canvas window
        self.map_dataCanvas = FigureCanvasTkAgg(self.map_fig, self.basemap_window)
        self.map_dataCanvas.get_tk_widget().pack(in_=self.map_display, side="bottom", fill="both", expand=1)
        self.map_toolbar = NavigationToolbar2Tk(self.map_dataCanvas, self.basemap_window)
        self.map_dataCanvas._tkcanvas.pack()
        self.basemap_state = 1
        self.draw_cid = self.map_fig.canvas.mpl_connect("draw_event", self.update_bg)
        self.pick_cid = self.map_fig.canvas.mpl_connect("pick_event", self.on_pick)


    def ontop(self):
        # toggle whether or not window is on top of all other windows
        self.basemap_window.wm_attributes("-topmost", str(self.ontop_bool.get()).lower())


    def set_vars(self):
        # initialize arrays to hold track nav info
        self.x = np.array(())
        self.y = np.array(())
        self.track_name = np.array(()).astype(str)
        self.loaded_tracks = np.array(()).astype(str)
        self.start_x = np.array(())
        self.start_y = np.array(())
        self.end_x = np.array(())
        self.end_y = np.array(())
        self.legend = None
        self.pick_loc = None
        self.profile_track = None


    # map is a method to plot the basemap in the basemap window
    def map(self, map_path):
        # pull track up on dem basemap
        if map_path:
            print("Loading Basemap: ", map_path.split("/")[-1])
            try:
                # read in basemap
                with rio.open(map_path, mode="r") as dataset:
                    # downsample if raster is too large
                    fac = 1
                    h = dataset.height
                    w = dataset.width
                    dx = dataset.transform[0]
                    if h >= 1e3 or w >= 1e3:
                        if dx < 10:
                            fac = np.amin([h*1e-3,w*1e-3])
                        else:
                            fac = 2

                    data = dataset.read(
                        out_shape=(dataset.count, int(h // fac), int(w// fac)),
                        resampling=rio.enums.Resampling.nearest
                    )
                    self.bmcrs = dataset.crs

                # stack bands into numpy array
                im = np.dstack((data))
                if im.shape[-1] == 1:
                    im = im.reshape(im.shape[0], -1)
                cmin = np.nanmin(im)
                cmax = np.nanmax(im)
                self.bm_im.set_clim([cmin, cmax])
                self.bm_im.set_data(im)
                # set bm image extent based on raster bounds - convert from m to km
                self.bm_im.set_extent([_i*1e-3 for _i in[dataset.bounds.left, dataset.bounds.right,
                dataset.bounds.bottom, dataset.bounds.top]])
                # save un-zoomed view to toolbar
                self.map_toolbar.push_current()
                self.map_fig_ax.set_visible(True)
                self.map_dataCanvas.draw()

            except Exception as err:
                print("basemap load error: " + str(err))
                pass


    # set_track is a method to pass the track name loaded in profile view to the basemap class
    def set_track(self, fn):
        self.profile_track = fn


    # set_nav is a method to update the navigation data plotted on the basemap
    def set_nav(self, fn, navdf):
        # skip if track already loaded to basemap
        if fn in self.track_name:
            return

        # transform navcrs to basemap crs
        xformer = Transformer.from_crs(crs_from=self.navcrs, crs_to=self.bmcrs.to_wkt())
        x, y = xformer.transform(
            navdf["lon"].to_numpy(),
            navdf["lat"].to_numpy(),
        )
    
        # convert from m to km
        x = x*1e-3
        y = y*1e-3
        # add x and y coords to x and y coord arrays
        self.x = np.append(self.x, x)
        self.y = np.append(self.y, y)
        # add name to array to match with x,y
        self.track_name = np.append(self.track_name, np.repeat(fn, len(x)))
        self.start_x = np.append(self.start_x, x[0])
        self.start_y = np.append(self.start_y, y[0])
        self.end_x = np.append(self.end_x, x[-1])
        self.end_y = np.append(self.end_y, y[-1])
        # add name to list to match with endpoints
        self.loaded_tracks = np.append(self.loaded_tracks, fn)


    # plot_tracks is a method to plot track geom
    def plot_tracks(self):
        buff = 1
        # if track_viz variable is true, add all track points to appropriate lines
        if self.track_viz.get():
            # set track line data
            self.track_ln.set_data(self.x, self.y)
            # set track ending line data
            self.track_start_ln.set_data(self.start_x, self.start_y)
            self.track_end_ln.set_data(self.end_x, self.end_y)

            x_range = [np.amin(self.x)- buff,np.amax(self.x)+ buff]
            y_range = [np.amin(self.y)- buff,np.amax(self.y)+ buff]

        # otherwise just set track points from last line to appropriate lines
        else:
            # set track ending line data
            idx = np.where(self.loaded_tracks == self.profile_track)[0]
            self.track_start_ln.set_data(self.start_x[idx], self.start_y[idx])
            self.track_end_ln.set_data(self.end_x[idx], self.end_y[idx])
            # set track line data
            idx = np.where(self.track_name == self.profile_track)[0]
            self.track_ln.set_data(self.x[idx], self.y[idx])
    
            x_range = [np.amin(self.x[idx])- buff,np.amax(self.x[idx])+ buff]
            y_range = [np.amin(self.y[idx])- buff,np.amax(self.y[idx])+ buff]
        r = max(x_range[1]-x_range[0],y_range[1]-y_range[0])
        xl = np.median(x_range) - r
        xr = np.median(x_range) + r
        yb = np.median(y_range) - r
        yt = np.median(y_range) + r
        # if track falls within bm, zoom in
        xaxis = self.map_fig_ax.get_xlim()
        yaxis = self.map_fig_ax.get_ylim()      
        # if (xl > xaxis[0]) & (xr < xaxis[1]) & (yb > yaxis[0]) & (yt < yaxis[1]):
        self.map_fig_ax.axis([xl,xr,yb,yt])

        if not self.legend:
            self.legend = self.map_fig_ax.legend()
        self.map_dataCanvas.draw() 
        self.blit()


    # plot_idx is a method to plot the location of a click event on the datacanvas to the basemap
    def plot_idx(self, fn, idx):
        # basemap open, plot picked location regardless of picking state
        if self.basemap_state == 1 and len(self.x) > 0:
            # plot pick location on basemap
            if self.pick_loc:
                self.pick_loc.remove()
            self.pick_loc = self.map_fig_ax.scatter(self.x[self.track_name == fn][idx], 
                                                    self.y[self.track_name == fn][idx], 
                                                    c="b", marker="X", zorder=3)
            self.blit()

        
    # basemap_close is a method to close the basemap window
    def basemap_close(self, event=None):
        self.basemap_window.destroy()
        self.basemap_state = 0


    # get_state is a mathod to get the basemap state
    def get_state(self):
        return self.basemap_state

    
    # clear_basemap is a method to clear the basemap 
    def clear_nav(self):
        # clear arrays
        self.track_name = np.array(()).astype(str)
        self.loaded_tracks = np.array(()).astype(str)
        self.x = np.array(())
        self.y = np.array(())
        self.start_x = np.array(())
        self.start_y = np.array(())
        self.end_x = np.array(())
        self.end_y = np.array(())
        # set lines
        self.track_ln.set_data(self.x, self.y)
        self.track_start_ln.set_data(self.start_x, self.start_y)
        self.track_end_ln.set_data(self.end_x, self.end_y)

        if self.legend:
            self.legend.remove()
            self.legend = None

        if self.pick_loc:
            self.pick_loc.remove()
            self.pick_loc = None

        self.map_fig.canvas.draw()


    def safe_draw(self):
        """temporarily disconnect the draw_event callback to avoid recursion"""
        canvas = self.map_fig.canvas
        canvas.mpl_disconnect(self.draw_cid)
        canvas.draw()
        self.draw_cid = canvas.mpl_connect("draw_event", self.update_bg)


    def update_bg(self, event=None):
        """
        when the figure is resized, hide picks, draw everything,
        and update the background.
        """
        if self.pick_loc:
            self.pick_loc.set_visible(False)
            self.safe_draw()
            self.axbg = self.map_dataCanvas.copy_from_bbox(self.map_fig_ax.bbox)
            self.pick_loc.set_visible(True)
            self.blit()
        else:
            self.safe_draw()
            self.axbg = self.map_dataCanvas.copy_from_bbox(self.map_fig_ax.bbox)
            self.blit()


    def blit(self):
        """
        update the figure, without needing to redraw the
        "axbg" artists.
        """
        self.map_fig.canvas.restore_region(self.axbg)
        if self.pick_loc:
            self.map_fig_ax.draw_artist(self.pick_loc)
        self.map_fig.canvas.blit(self.map_fig_ax.bbox)


    # load all tracks within a directory to basemap
    def load_tracks(self, dir = False):
        if dir:
            tmp_datPath = ""
            # select input file
            tmp_datPath = tk.filedialog.askdirectory(title="select folder",
                                        initialdir=self.datPath,
                                        mustexist=True) 
            # if input selected, clear impick canvas, ingest data and pass to impick
            if tmp_datPath:
                # get list of data files in dir
                flist = glob.glob(tmp_datPath + "/*")
                print(tmp_datPath)

        else: 
            flist = ""
            # select input files
            flist = tk.filedialog.askopenfilenames(title="select files",
                                        initialdir=self.datPath, 
                                        multiple=True) 
            if flist:
                flist = list(flist)

            # iterate through file list and retrieve navdat from proper getnav function
        for f in flist:

            if f.endswith("h5"):
                navdf = navparse.getnav_oibAK_h5(f, self.navcrs, self.body)
                fn = f.split("/")[-1].rstrip(".h5")
            elif f.endswith("mat"):
                try:
                    navdf = navparse.getnav_cresis_mat(f, self.navcrs, self.body)
                    fn = f.split("/")[-1].rstrip(".mat")
                except:
                    navdf = navparse.getnav_oibAK_h5(f, self.navcrs, self.body)
                    fn = f.split("/")[-1].rstrip(".mat")
            elif f.lower().endswith("dzg"):
                navdf = navparse.getnav_gssi(f, self.navcrs, self.body)
                fn = f.split("/")[-1].rstrip(".DZG")
            elif f.endswith("tab"):
                navdf = navparse.getnav_sharad(f, self.navcrs, self.body)
                fn = f.split("/")[-1].rstrip("_geom.tab")
            else:
                continue
            self.set_nav(fn, navdf)

        # update datPath
        self.datPath = os.path.dirname(os.path.abspath(f)) + "/"
        # update extent
        self.plot_tracks()


    # settings menu
    def settings(self):
        settingsWindow = tk.Toplevel(self.basemap_window)

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=10, text="cmap", anchor="w")
        lab.pack(side=tk.LEFT)
        tk.Radiobutton(row,text="greys_r", variable=self.cmap, value="Greys_r").pack(side="top",anchor="w")
        tk.Radiobutton(row,text="gray", variable=self.cmap, value="gray").pack(side="top",anchor="w")
        tk.Radiobutton(row,text="terrain", variable=self.cmap, value="terrain").pack(side="top",anchor="w")

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, anchor="c")
        b1 = tk.Button(row, text="save",
                    command=self.updateSettings)
        b1.pack(side="left")
        b2 = tk.Button(row, text="close", command=settingsWindow.destroy)
        b2.pack(side="left")


    # update settings
    def updateSettings(self):
        # pass cmap
        self.bm_im.set_cmap(self.cmap.get())
        # redraw
        self.map_dataCanvas.draw() 


    # on_pick gets the picked track from user click
    def on_pick(self, event):
        # determine which track was selected
        xdata = event.mouseevent.xdata
        idx = utils.find_nearest(self.x, xdata)
        track = self.track_name[idx]
        # pass track to impick
        if (track != self.profile_track) and (tk.messagebox.askyesno("Load","Load track: " + str(track) + "?") == True):
            self.to_gui(self.datPath, track)


# RAGU - Radar Analysis Graphical Utility
#
# copyright © 2020 btobers <tobers.brandon@gmail.com>
#
# distributed under terms of the GNU GPL3.0 license
"""
RAGU - Radar Analysis Graphical Utility
created by: Brandon S. Tober and Michael S. Christoffersen
date: 25JUN19
last updated: 08JUL2020
environment requirements in ragu.yml
"""
### imports ###
from ragu import config
from ragu.ui import gui
import os,sys,argparse
import tkinter as tk
from tkinter import font

def main():

    # get configuration file
    basedir = os.path.join(os.path.expanduser('~'),'RAGU')
    if not os.path.isdir(basedir):
        os.mkdir(basedir)
    if not os.path.isfile(basedir+'/config.ini'):
        config.create_config(basedir+'/config.ini')
    configPath = basedir + '/config.ini'

    # set up CLI
    parser = argparse.ArgumentParser(
    description=f"Radar Analysis Graphical Utility (RAGU)\n\nFor documentation see: https://github.com/btobers/RAGU\nDefulat configuration file path: {configPath}",
        formatter_class=argparse.RawTextHelpFormatter
    )

    parser.add_argument("-datPath", help="Data directory file path", nargs="?", default='')
    parser.add_argument("-datFile", help="Data file path to open load", nargs="?", default='')
    parser.add_argument("-configPath", help="Configuration file path", nargs="?", default=configPath)
    args = parser.parse_args()
    datPath = args.datPath
    datFile = args.datFile
    tmp = args.configPath
    # check if path exists
    if datPath:
        if not os.path.isdir(datPath):
            print(f"Data directory not found : {datPath}")
            print(f"Defaulting to data file path specified in the RAGU configuration file.")
            datPath = None
    if os.path.isfile(tmp) and tmp.endswith(".ini"):
        configPath=tmp
    else:
        print(f"Configuration file not found at: {tmp}\nFull file path must be entered.\nDefaulting to: {configPath}")

    # check if dataFile exists
    if datFile and not os.path.isfile(datFile):
        print(f"Data file does not exist: {datFile}")

    # change dir to RAGU code directory 
    os.chdir(os.path.dirname(os.path.abspath(__file__)))

    # initialize tkinter
    root = tk.Tk()

    # get screen size - open root window half screen
    w, h = root.winfo_screenwidth(), root.winfo_screenheight()
    root.geometry("%dx%d+0+0" % (.5*w, .5*h))
    root.title("RAGU")
    root.config(bg="#d9d9d9")
    try:
        for _i in tk.font.names():
            # tk.font.nametofont(_i).config(family="Times New Roman", size=10, weight="normal")
            tk.font.nametofont(_i).config(size=10, weight="normal")
    except:
        pass

    # call the RAGU mainGUI class
    gui.mainGUI(root, configPath=configPath, datPath=datPath, datFile=datFile)
    root.lift()    
    root.mainloop()

if __name__ == "__main__":
    main()


# RAGU - Radar Analysis Graphical Utility
#
# copyright © 2020 btobers <tobers.brandon@gmail.com>
#
# distributed under terms of the GNU GPL3.0 license
"""
ingest_oibAK is a module developed to ingest NASA OIB-AK radar sounding data. 
primary data format is hdf5, however some older data is still being converted over from .mat format
"""
### imports ###
from ragu.radar import garlic
from ragu.nav import navparse
from ragu.tools import utils
import h5py, fnmatch
import numpy as np
import scipy as sp
import sys

# method to ingest OIB-AK radar hdf5 data format
def read_h5(fpath, navcrs, body):
    rdata = garlic(fpath)
    rdata.fn = fpath.split("/")[-1][:-3]
    rdata.dtype = "oibak"

    # read in .h5 file
    f = h5py.File(rdata.fpath, "r")                      

    # h5 radar data group structure        
    # |-raw
    # |  |-rx0
    # |  |-tx0    
    # |  |-loc0
    # |-ext
    # |  |-nav0
    # |  |-srf0
    # |-drv
    # |  |-proc0
    # |  |-clutter0
    # |  |-pick

    # pull necessary raw group data
    rdata.snum = int(f["raw"]["rx0"].attrs["samplesPerTrace"])                  # samples per trace in rgram
    rdata.tnum = int(f["raw"]["rx0"].attrs["numTrace"])                         # number of traces in rgram 
    rdata.fs = f["raw"]["rx0"].attrs["samplingFrequency"][0]                    # sampling frequency
    rdata.dt = 1/rdata.fs                                                       # sampling interval, sec
    rdata.prf = f["raw"]["tx0"].attrs["pulseRepetitionFrequency"][0]            # pulse repition frequency, Hz
    rdata.nchan = 1

    # pull radar proc and sim arrayss
    rdata.set_dat(f["drv/proc0"][:])                                            # pulse compressed array
    rdata.set_proc(np.abs(rdata.get_dat()))
    if "clutter0" in f["drv"].keys():
        rdata.set_sim(f["drv"]["clutter0"][:])                                  # simulated clutter array

    rdata.set_twtt()
    # assign signal info
    rdata.info["Signal Type"] = f["raw"]["tx0"].attrs["signal"].capitalize() 
    rdata.info["CF [MHz]"] = f["raw"]["tx0"].attrs["centerFrequency"][0] * 1e-6
    if rdata.info["Signal Type"] == "Chirp":
        rdata.info["Badwidth [%]"] = f["raw"]["tx0"].attrs["bandwidth"][0] * 100
        rdata.info["Pulse Length [\u03BCs]"] = f["raw"]["tx0"].attrs["length"][0] * 1e6
    rdata.info["Sampling Frequency [MHz]"] = rdata.fs * 1e-6
    rdata.info["PRF [kHz]"] = rdata.prf * 1e-3

    # parse nav
    rdata.geocrs = navcrs
    rdata.xyzcrs = navparse.xyzsys[body]
    rdata.navdf = navparse.getnav_oibAK_h5(fpath, navcrs, body)
    # pull lidar surface elevation and initilize horizon
    if "srf0" in f["ext"].keys():
        rdata.set_srfElev(dat = f["ext"]["srf0"][:])  
        if "twtt_surf" in f["drv"]["pick"].keys():
            twtt_srf = f["drv"]["pick"]["twtt_surf"][:]
            # replace -1 and -9 null values with np.nan
            twtt_srf[twtt_srf == -1] = np.nan
            twtt_srf[twtt_srf == -9] = np.nan
            arr = utils.twtt2sample(twtt_srf, rdata.dt)
        else:
            arr = utils.twtt2sample(utils.depth2twtt(rdata.navdf["elev"] - rdata.srfElev, rdata.asep, eps_r=1), rdata.dt)
    else:
        arr = np.repeat(np.nan, rdata.tnum)
        rdata.set_srfElev(dat = arr)
    rdata.pick.horizons["srf"] = arr
    rdata.pick.set_srf("srf")

    # # read in bed picks
    # if "twtt_bed" in f["drv"]["pick"].keys():
    #     twtt_bed = f["drv"]["pick"]["twtt_bed"][:]
    #     # replace -1 and -9 null values with np.nan
    #     twtt_bed[twtt_bed == -1] = np.nan
    #     twtt_bed[twtt_bed == -9] = np.nan
    #     arr = utils.twtt2sample(twtt_bed, rdata.dt)
    # else:
    #     arr = np.repeat(np.nan, rdata.tnum)
    # rdata.pick.horizons["bed"] = arr
    

    f.close()                                                   # close the file

    rdata.check_attrs()

    return rdata

# method to ingest .mat files OIB-AK. for older matlab files, sp.io seems to work while h5py does not. for newer files, h5py seems to work while sp.io does not 
def read_mat(fpath, navcrs, body):
    fn = fpath.split("/")[-1]
    print("----------------------------------------")
    print("Loading: " + fn)
    rdata = garlic(fpath)
    rdata.fn = fn[:-4]
    rdata.dtype = "oibak"
    # read in .mat file
    try:
        f = h5py.File(rdata.fpath, "r")
        rdata.snum = int(f["block"]["num_sample"][0])[-1]
        rdata.tnum = int(f["block"]["num_trace"][0])[-1] 
        rdata.dt = float(f["block"]["dt"][0])*2
        rdata.dat = np.array(f["block"]["amp"])
        rdata.set_proc(rdata.dat)

        rdata.navdf = navparse.getnav_oibAK_mat(fpath, navcrs, body)
        rdata.set_sim(np.array(f["block"]["clutter"]))
        twtt_srf = f["block"]["twtt_surf"].flatten()

        f.close()

    except:
        try:
            f = sp.io.loadmat(rdata.fpath)
            rdata.snum = int(f["block"]["num_sample"][0])
            rdata.tnum = int(f["block"]["num_trace"][0])
            rdata.dt = float(f["block"]["dt"][0])*2
            rdata.dat = f["block"]["amp"][0][0]
            rdata.set_proc(rdata.dat)

            rdata.navdf = navparse.getnav_oibAK_mat(fpath, navcrs, body)
            rdata.set_sim(f["block"]["clutter"][0][0])
            twtt_srf = f["block"]["twtt_surf"][0][0].flatten()

        except Exception as err:
            print("ingest Error: " + str(err))
            pass

    rdata.set_twtt()

    if not np.isnan(twtt_srf).all():
        arr = utils.twtt2sample(twtt_srf, rdata.dt)
        rdata.pick.horizons["srf"] = arr
        rdata.pick.srf = "srf"

        # get surface elevation
        arr = rdata.navdf["elev"] - utils.twtt2depth(twtt_srf, rdata.asep, eps_r=1)
        rdata.set_srfElev(dat = arr)

    rdata.check_attrs()

    return rdata


# RAGU - Radar Analysis Graphical Utility
#
# copyright © 2020 btobers <tobers.brandon@gmail.com>
#
# distributed under terms of the GNU GPL3.0 license
"""
ingest_marsis_ipc is a module developed to ingest MARSIS radar sounding data processed by Michael Christoffersen. 
data format is binary 32-bit floating point pulse compressed data
"""
### imports ###
from ragu.radar import garlic
from ragu.nav import navparse
from ragu.tools import utils
from PIL import Image
import numpy as np
import os, sys, glob
import matplotlib.pyplot as plt

# method to read JPL multilook MARSIS data
def read(fpath, simpath, navcrs, body):
    rdata = garlic(fpath)
    rdata.fn = fpath.split("/")[-1][:-4]
    rdata.dtype = "marsis_ipc"
    root = os.path.dirname(fpath)

    # convert binary RGRAM to numpy array
    # # reshape array, knowing that each trace has 512 samples
    dtype = np.dtype("float32")     
    with open(fpath, "rb") as f:
        tmp = np.fromfile(f, dtype)     
    l = len(tmp)

    rdata.snum = 512
    rdata.tnum = int(len(tmp)/rdata.snum)
    rdata.fs = 1.4e6
    rdata.dt = rdata.fs
    rdata.prf = 127
    rdata.nchan = 1
    rdata.set_dat(tmp.reshape((rdata.snum,rdata.tnum), order='F'))
    rdata.set_proc(rdata.get_dat())

    # convert binary .img clutter sim product to numpy array
    if simpath:
        simpath = simpath + "/" + rdata.fn + "_geom_combined.img"
    else:
        simpath = root + "/" + rdata.fn + "_geom_combined.img"

    if os.path.isfile(simpath):
        with open(simpath, "rb") as f:
            sim = np.fromfile(f, dtype)   
        sim = sim.reshape(rdata.snum,rdata.tnum)
        rdata.set_sim(sim)
    else:
        print("Clutter simulation not found:\t{}\nSpecify alternate path in configuration file.".format(simpath))

    rdata.set_twtt()

    # assign signal info
    rdata.info["dataset"] = "marsis_ipc"
    # rdata.info["signal type"] = "chirp"
    # rdata.info["cf [MHz]"] = 20
    # rdata.info["badwidth [%]"] = 50
    # rdata.info["pulse length [\u03BCs]"] = 85
    rdata.info["sampling frequency [MHz]"] = rdata.fs * 1e-6
    rdata.info["prf [Hz]"] = rdata.prf

    # open geom nav file for rgram
    geom_path = root +  "/" + rdata.fn[:-2] + "nav.csv"

    # parse nav
    rdata.geocrs = navcrs
    rdata.xyzcrs = navparse.xyzsys[body]
    rdata.navdf = navparse.getnav_marsis_ipc(geom_path, navcrs, body)

    rdata.set_srfElev(dat = np.repeat(np.nan, rdata.tnum))

    rdata.check_attrs()

    return rdata


# RAGU - Radar Analysis Graphical Utility
#
# copyright © 2020 btobers <tobers.brandon@gmail.com>
#
# distributed under terms of the GNU GPL3.0 license
"""
rdata pick ojects
"""
import numpy as np

class pick(object):
    """
    pick class holds the relevant radar pick information.
    """
    def __init__(self):
        #: dict horizons, nested containing  file pick horizons
        # each horizon is itself a dictionary with each key 
        # of type np.ndarray(tnum,), representing the
        # pick in sample number for each trace
        self.horizons = {}
        #: str srf, surface horizon name
        self.srf = None

        return


    # set_srf defines the surface horizon name
    def set_srf(self, srf=None):
        self.srf = srf


    # get_srf returns the defined surface horizon name
    def get_srf(self):
        return self.srf


    # get_pick_flag returns true if interpretations exist, false otherwise
    def get_pick_flag(self):
        flag = False
        for key, item in self.horizons.items():
            if np.isnan(item).all():
                continue
            else:
                flag = True
                break

        return flag


# RAGU - Radar Analysis Graphical Utility
#
# copyright © 2020 btobers <tobers.brandon@gmail.com>
#
# distributed under terms of the GNU GPL3.0 license
"""
notepad class is a tkinter frame which handles the RAGU session notes
"""

import tkinter as tk
import os, sys
  
class notepad(tk.Frame):

    # default window width and height 
    __thisWidth = 600
    __thisHeight = 300      
    __file = ""

    def __init__(self, parent, init_dir, **kwargs):

        self.__parent = parent
        self.__init_dir = init_dir
        self.__state = 0
        try: 
            self.__thisWidth = kwargs['width'] 
        except KeyError: 
            pass
  
        try: 
            self.__thisHeight = kwargs['height'] 
        except KeyError: 
            pass
    
    def __setup(self, path=""):
        # create tkinter toplevel window to display note
        self.__root = tk.Toplevel(self.__parent)
        self.__root.config(bg="#d9d9d9")
        self.__root.title("RAGU - Session Notes")
        self.__root.bind("<Control-q>", self.__quit)
        self.__root.bind("<Control-s>", self.__saveFile)
        self.__root.bind("<Control-o>", self.__openFile)
        self.__root.bind("<Control-n>", self.__newFile)
        self.__root.protocol("WM_DELETE_WINDOW", self.__quit)
        self.__thisMenuBar = tk.Menu(self.__root) 
        self.__thisFileMenu = tk.Menu(self.__thisMenuBar, tearoff=0) 
        self.__thisEditMenu = tk.Menu(self.__thisMenuBar, tearoff=0) 
        self.__thisHelpMenu = tk.Menu(self.__thisMenuBar, tearoff=0) 
        self.__thisTextArea = tk.Text(self.__root, font=("Times New Roman", 14)) 
        self.__thisScrollBar = tk.Scrollbar(self.__thisTextArea)

        # Center the window 
        screenWidth = self.__root.winfo_screenwidth() 
        screenHeight = self.__root.winfo_screenheight() 
      
        # For left-alling 
        left = (screenWidth / 2) - (self.__thisWidth / 2)  
          
        # For right-allign 
        top = (screenHeight / 2) - (self.__thisHeight /2)  
          
        # For top and bottom 
        self.__root.geometry('%dx%d+%d+%d' % (self.__thisWidth, 
                                              self.__thisHeight, 
                                              left, top))  
  
        # To make the textarea auto resizable 
        self.__root.grid_rowconfigure(0, weight=1) 
        self.__root.grid_columnconfigure(0, weight=1) 
  
        # Add controls (widget) 
        self.__thisTextArea.grid(sticky = tk.N + tk.E + tk.S + tk.W) 
          
        # To open new file 
        self.__thisFileMenu.add_command(label="New", 
                                        command=self.__newFile)     
          
        # To open a already existing file 
        self.__thisFileMenu.add_command(label="Open", 
                                        command=self.__openFile) 
          
        # To save current file 
        self.__thisFileMenu.add_command(label="Save", 
                                        command=self.__saveFile)     
  
        # To create a line in the dialog         
        self.__thisFileMenu.add_separator()                                          
        self.__thisFileMenu.add_command(label="Exit", 
                                        command=self.__quit) 
        self.__thisMenuBar.add_cascade(label="File", 
                                       menu=self.__thisFileMenu)      
          
        # To give a feature of cut  
        self.__thisEditMenu.add_command(label="Cut", 
                                        command=self.__cut)              
      
        # to give a feature of copy     
        self.__thisEditMenu.add_command(label="Copy", 
                                        command=self.__copy)          
          
        # To give a feature of paste 
        self.__thisEditMenu.add_command(label="Paste", 
                                        command=self.__paste)          
          
        # To give a feature of editing 
        self.__thisMenuBar.add_cascade(label="Edit", 
                                       menu=self.__thisEditMenu) 
             
          # To create a feature of description of the notepad 
        self.__thisHelpMenu.add_command(label="About Notepad", 
                                        command=self.__showAbout)  
        self.__thisMenuBar.add_cascade(label="Help", 
                                       menu=self.__thisHelpMenu) 
                            
        self.__root.config(menu=self.__thisMenuBar) 
  
        self.__thisScrollBar.pack(side=tk.RIGHT,fill=tk.Y)                     
          
        # Scrollbar will adjust automatically according to the content         
        self.__thisScrollBar.config(command=self.__thisTextArea.yview)      
        self.__thisTextArea.config(yscrollcommand=self.__thisScrollBar.set)
        self.__set_state(1)
        if path:
            self.__file = path
        self.__header()
      
          
    def __quit(self, event=None):
        # check if text exists and changes made since last save
        if len(self.__get_text()) > 1:
            if self.__file:
                f = open(self.__file,"r")
                text = f.read().strip()
                if (self.__get_text().strip() != text) and not (tk.messagebox.askyesno("Warning", "Exit notepad without saving?", icon = "warning")):
                    return
        self.__root.destroy() 
        self.__set_state(0)

    def __openFile(self, event=None): 
        if not self.__file:
            self.__file = tk.filedialog.askopenfilename(initialfile='Untitled.csv',
                                                        initialdir=self.__init_dir,
                                                        defaultextension=".csv", 
                                                        filetypes=[("Comma-separated value","*.csv"),
                                                        ("All Files","*.*")]) 

        if self.__file == "": 
            # no file to open 
            self.__file = None
        else: 
            # Try to open the file 
            # set the window title 
            self.__root.title(os.path.basename(self.__file) + " - RAGU Notepad") 
            self.__thisTextArea.delete(1.0,tk.END) 

            file = open(self.__file,"r") 

            self.__thisTextArea.insert(1.0,file.read().strip()) 

            file.close() 

            # self.__header()

    def __header(self):
        header = "track,bed (y/n/?),confidence (h/m/l),note"
        if self.__get_text().splitlines()[0] != header:
            self.__thisTextArea.insert(tk.END, header + "\n")
            self.__thisTextArea.see("insert")

    def __newFile(self, event=None):
        if len(self.__get_text()) > 1:
            if not tk.messagebox.askyesno("Warning", "Discard changes?", icon = "warning"):
                return
        self.__root.title("Untitled - RAGU Notepad") 
        self.__file = None
        self.__thisTextArea.delete(1.0,tk.END) 

    def __saveFile(self, event=None): 
        # if not file path exists, or path does exist but overwrite not allowed, have user enter new path
        if not self.__file:
            # get file name
            f = tk.filedialog.asksaveasfilename(initialfile='Untitled.csv',
                                                        initialdir=self.__init_dir,
                                                        defaultextension=".csv",
                                                        filetypes=[("Comma-separated value","*.csv"),
                                                        ("All Files","*.*")])

            if f:
                self.__file = f
                flag = 1
        
        # if file does exist
        elif self.__file:
            flag = 0

        else:
            return

        # save
        file = open(self.__file,"w")
        file.write(self.__thisTextArea.get(1.0,tk.END).strip()) 
        file.close() 

        # Change the window title 
        self.__root.title(os.path.basename(self.__file) + " - RAGU Notepad") 
        if flag:
            print("Notepad saved successfully:\t{}".format(self.__file))
                  

    def __showAbout(self): 
        tk.messagebox.showinfo("RAGU Notepad","Notepad designed to hold session notes. Radar data file names will be added to new notepad line in a comma-separated value format. Type file notes following file name.") 

    # __write_track adds the current track from gui to a new line
    def __write_track(self, fn=None):
        if fn:
            text = self.__get_text()
            if fn in text:
                self.__search_text(fn)
                start = self.__thisTextArea.index("insert linestart")

            else:
                if len(text) > 1:
                    lline = text.rsplit("\n")[-2]
                    new = ""
                    while len(lline.split(",")) < 3:
                        new += ","
                        lline += new
                    self.__thisTextArea.insert(tk.END, new)
                    self.__thisTextArea.see("insert")
                    fn = "\n" + fn

                self.__thisTextArea.insert(tk.END, fn + ",")
                self.__thisTextArea.see("insert")
                start = self.__thisTextArea.index("insert linestart")

            # highlight current line
            end = self.__thisTextArea.index(start.split(".")[0] + "." + str(len(fn)))
            self.__highlighter(start, end)


    # __search_text
    def __search_text(self, lookup):
        for num, line in enumerate(self.__get_text().splitlines()):
            if lookup in line:
                self.__thisTextArea.mark_set("insert", str(num+1) + "." + str(len(line)))
                self.__thisTextArea.see("insert")


    def __highlighter(self, start, end):
        if all(v is not None for v in [start, end]):  
            # remove previous tag
            idx_prev = self.__thisTextArea.tag_ranges("highlight")
            if len(idx_prev) > 0:
                self.__thisTextArea.tag_remove("highlight", idx_prev[0], idx_prev[1])
            self.__textHighlight = self.__thisTextArea.tag_add("highlight", start, end)
            self.__thisTextArea.tag_config("highlight", background="yellow")


    # __get_text returns the text entry string
    def __get_text(self):
        return self.__thisTextArea.get(1.0, tk.END)
  
    def __cut(self): 
        self.__thisTextArea.event_generate("<<Cut>>") 
  
    def __copy(self): 
        self.__thisTextArea.event_generate("<<Copy>>")
  
    def __paste(self): 
        self.__thisTextArea.event_generate("<<Paste>>") 
  
    def __get_state(self):
        return self.__state

    def __set_state(self, state=0):
        self.__state = state

    def __get_file(self):
        return self.__file



# RAGU - Radar Analysis Graphical Utility
#
# copyright © 2020 btobers <tobers.brandon@gmail.com>
#
# distributed under terms of the GNU GPL3.0 license
"""
this module contains functions parsed from https://github.com/iannesbitt/readgssi to read gssi dzt files and associated dzg files for use in RAGU
much of the header data which is not necessary for RAGU use has been removed
"""
### imports ###
from ragu.radar import garlic
from ragu.nav import navparse
from ragu.tools.constants import *
import struct
import os,sys
import numpy as np

# method to read gssi dzt data
def read(fpath, navcrs, body):
    fn = fpath.split("/")[-1]
    rdata = garlic(fpath)
    rdata.fn = fn[:-4]
    rdata.dtype = "gssi"

    # modified readgssi readdzt.dzt reader (credit, DOI: 10.5281/zenodo.3352438)
    with open(rdata.fpath, "rb") as f:

        # skip around and read necessary attributes
        f.seek(2)
        data_offset = struct.unpack("<h", f.read(2))[0]     # offset to data array [bits]
        rdata.snum = struct.unpack('<h', f.read(2))[0]      # number of samples per trace
        bits = struct.unpack('<h', f.read(2))[0]            # number of bits - datatype
        f.seek(10)
        rdata.prf = struct.unpack('<f', f.read(4))[0]       # scans per second (prf)
        f.seek(26)
        range_ns = struct.unpack('<f', f.read(4))[0]        # data range [ns] - record time per trace
        f.seek(52)
        rdata.nchan = struct.unpack('<h', f.read(2))[0]     # number of data channels
        rdata.dt = range_ns / rdata.snum * 1.0e-9           # sampling interval
        rdata.fs = 1/rdata.dt                               # sampling freq

        if bits == 8:
            dtype = np.uint8    # 8-bit unsigned
        elif bits == 16:
            dtype = np.uint16   # 16-bit unsigned
        elif bits == 32:
            dtype = np.int32    # 32-bit signed
        else:
            print("ingest_gssi.read error: undefined data type. #bits = " + bits)
            exit(1)

        # skip ahead to data
        if data_offset < 1024: # whether or not the header is normal or big-->determines offset to data array
            didx = 1024 * data_offset
        else:
            didx = 1024 * rdata.chan

        # skip to data index
        f.seek(didx)

        # read in data - need to transpose to get correct shape
        rdata.set_dat(np.fromfile(f, dtype).reshape(-1,(rdata.snum*rdata.nchan)).T)

    # ensure data file is not empty
    if not np.any(rdata.dat):
        raise ValueError("gssi_read error: file contains no radar data")

    rdata.tnum = rdata.dat.shape[1]

    # convert gssi signed int amplitude to floating point for displaying
    rdata.set_proc(rdata.get_dat().astype(float))

    rdata.set_twtt()

    # assign signal info
    rdata.info["Signal Type"] = "Impulse"
    rdata.info["Sampling Frequency [MHz]"] = rdata.fs * 1e-6
    rdata.info["PRF [kHz]"] = rdata.prf * 1e-3

    # read in gps data if exists
    infile_gps = fpath.replace(".DZT",".DZG")

    # create nav object to hold lon, lat, hgt
    rdata.geocrs = navcrs
    rdata.xyzcrs = navparse.xyzsys[body]
    rdata.navdf = navparse.getnav_gssi(infile_gps, rdata.tnum, navcrs, body)

    # for ground-based GPR, elev_gnd is the same as GPS recorded elev
    rdata.set_srfElev(dat = rdata.navdf["elev"].to_numpy())

    rdata.check_attrs()

    return rdata


# RAGU - Radar Analysis Graphical Utility
#
# copyright © 2020 btobers <tobers.brandon@gmail.com>
#
# distributed under terms of the GNU GPL3.0 license
"""
ingest_rimfax is a module developed to ingest NASA's Perseverance Radar Imager for Mars' subsurface experiment (RIMFAX) data.
"""
### imports ###
from ragu.radar import garlic
from ragu.nav import navparse
from ragu.tools import utils
import h5py, fnmatch
import pandas as pd
import numpy as np
import sys
import matplotlib.pyplot as plt
# method to ingest RIMFAX radar data from the PDS

###
#  data files should first be split up into separate files for each mode of active sounding - use the following commented code to do this:
## mode=[26,78,214]
## name=['shallow','surface','deep']
## for fn in glob.glob('rimfax*.csv'):
##     f = pd.read_csv(fn, header=0)
##     for m,n in zip(mode,name):
##         f_ = f.loc[(f["record_type"]==0) & (f["config_id"]==m)]
##         f_.to_csv(fn[:-4]+'_'+n+'.csv')
####

def read(fpath, navcrs, body):
    rdata = garlic(fpath)
    rdata.fn = fpath.split("/")[-1][:-4]
    rdata.dtype = "rimfax"

    # only read necessary data from file
    f = pd.read_csv(rdata.fpath, header=0)
    f = f.loc[f["record_type"]==0]                                                      # only retain active sounding
    idx = f.columns.get_loc("s0001")
    dat = np.array(f.iloc[:,idx:])                                                      # parse actual data
    # find traces with all null samples
    filt = ~np.isnan(dat).all(axis=1)                                                   
    f = f.iloc[filt,:]
    dat = dat[filt,:]

    rdata.set_dat(np.array(dat).T)                                                      # transpose 
    rdata.set_proc(np.abs(rdata.get_dat()))

    rdata.snum, rdata.tnum =  dat.shape                                                 # snum, tnum
    rdata.dt = np.mean(f["sample_time_increment"])*1e-9                                 # sampling interval, sec - don't know if it's necessary to use the mean. i wouldn't expect the value doesn't change, but not sure

    rdata.prf = 1 / rdata.dt                                                           # pulse repitition frequency
    rdata.nchan = 1
    rdata.set_twtt()

    # parse nav
    rdata.geocrs = navcrs
    rdata.xyzcrs = navparse.xyzsys[body]
    rdata.navdf = navparse.getnav_rimfax(fpath, navcrs, body)
    rdata.set_srfElev(dat = np.repeat(np.nan, rdata.tnum))

    rdata.info["PRF [kHz]"] = rdata.prf * 1e-3

    rdata.check_attrs()
    return rdata


# RAGU - Radar Analysis Graphical Utility
#
# copyright © 2020 btobers <tobers.brandon@gmail.com>
#
# distributed under terms of the GNU GPL3.0 license
"""
ingest_lrs is a module developed to ingest JAXA KAGUYA (SELENE) Lunar Radar Sounder (LRS) data.
"""
### imports ###
from ragu.radar import garlic
from ragu.nav import navparse
from ragu.tools import utils
import numpy as np
import os, sys

# method to read KAGUYA (SELENE) LRS SAR data
def read(fpath, simpath, navcrs, body):
    rdata = garlic(fpath)
    rdata.fn = fpath.split("/")[-1][:-4]
    rdata.dtype = "lrs"
    root = os.path.dirname(fpath)

    # get number of traces (file records in lbl file) and samples per trace
    with open(fpath.replace(".img", ".lbl"),'r') as f:
        lbl = f.read().split('\n')
    rdata.tnum = int(lbl[19].split('=')[1])
    rdata.snum = 1000

    # convert binary .img RGRAM to numpy array - data begins after 55 bytes of header info
    with open(rdata.fpath, "rb") as f:
        f.seek(rdata.tnum*55)
        tmp = f.read(rdata.tnum*rdata.snum)
        dat = np.frombuffer(tmp, np.uint8)
    l = len(dat)

    rdata.dt = 305.17578125e-09
    rdata.prf = 20
    rdata.nchan = 1
    rdata.set_dat(dat.reshape(rdata.snum,rdata.tnum))
    rdata.set_proc(rdata.get_dat())
    
    # convert binary .img clutter sim product to numpy array
    if simpath:
        simpath = simpath + "/" + rdata.fn + "_geom_combined.img"
    else:
        simpath = root + "/" + rdata.fn + "_geom_combined.img"

    if os.path.isfile(simpath):
        with open(simpath, "rb") as f:
            sim = np.fromfile(f, np.float32)   
        sim = sim.reshape(rdata.snum,rdata.tnum)
        rdata.set_sim(sim)

    rdata.set_twtt()

    # assign signal info
    rdata.info = {}
    rdata.info["Signal Type"] = "Chirp"
    rdata.info["CF [MHz]"] = 5
    rdata.info["Bandwidth [%]"] = 40
    rdata.info["Pulse Length [\u03BCs]"] = 200
    rdata.info["PRF [Hz]"] = rdata.prf

    # open geom nav file for rgram - Michael Christoffersen post-processed geom files using SPICE trajectory. 
    # if this is not available, use binary stored nav.
    geom_path = fpath.replace(".lbl","_geom.csv")
    if not os.path.isfile(geom_path):
        geom_path = rdata.fpath

    # parse nav - use spice derived nav, not header data due to header data issues
    rdata.geocrs = navcrs
    rdata.xyzcrs = navparse.xyzsys[body]
    rdata.navdf = navparse.getnav_lrs(geom_path, navcrs, body, rdata.tnum)

    rdata.set_srfElev(dat = np.repeat(np.nan, rdata.tnum))

    rdata.check_attrs()

    return rdata


# RAGU - Radar Analysis Graphical Utility
#
# copyright © 2020 btobers <tobers.brandon@gmail.com>
#
# distributed under terms of the GNU GPL3.0 license
"""
RAGU - Radar Analysis Graphical Utility
created by: Brandon S. Tober and Michael S. Christoffersen
date: 25JUN19
last updated: 05FEB20
environment requirements in nose_env.yml

mainGUI class is a tkinter frame which runs the RAGU master GUI
"""
### imports ###
from ragu.raguError import raguError
from ragu.ui import impick, wvpick, basemap, notepad
from ragu.tools import utils, export
from ragu.ingest import ingest
import os, sys, scipy, glob, configparser, datetime, copy
import numpy as np
import pandas as pd
import matplotlib as mpl
mpl.use("TkAgg")
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
import tkinter as tk
import tkinter.ttk as ttk
# try:
#     plt.rcParams["font.family"] = "Times New Roman"
# except:
#     pass

class mainGUI(tk.Frame):
    def __init__(self, parent, *args, **kwargs):
        tk.Frame.__init__(self, parent)
        self.parent = parent
        # read and parse config
        self.conf = configparser.ConfigParser()
        self.conf.read(kwargs['configPath'])
        if kwargs['datPath']:
            self.datPath = kwargs['datPath']
        else:
            self.datPath = self.conf["path"]["datPath"]
        # initialize variables
        self.rdata = None
        self.f_loadName = ""
        self.map_loadName = ""
        self.tab = "Profile"
        self.eps_r = tk.DoubleVar(value=self.conf["output"]["eps_r"])
        self.popup = popup(self.parent)
        self.proj = project()
        self.pick_vis = tk.BooleanVar()
        self.pick_vis.set(True)
        self.ann_vis = tk.BooleanVar()
        self.ann_vis.set(True)
        # dictionary to hold figure settings
        self.figsettings = {"cmap": tk.StringVar(value=self.conf["param"]["cmap"]),
                            "figsize": tk.StringVar(value="6.5,1.5"), 
                            "fontsize": tk.DoubleVar(value="10"),
                            "figtitle": tk.BooleanVar(),
                            "figxaxis": tk.BooleanVar(),
                            "figyaxis": tk.BooleanVar(),
                            "figclip": (tk.DoubleVar(value=0.0),tk.DoubleVar(value=1.0))}
        self.figsettings["figxaxis"].set(True)        
        self.figsettings["figyaxis"].set(True)        
        self.figsettings["figtitle"].set(True)        
        self.debugState = tk.BooleanVar()
        self.debugState.set(False)
        self.os = sys.platform
        # setup tkinter frame
        self.setup()
        # if passed, load datafile
        if kwargs['datFile']:
            self.open_dfile(kwargs['datFile'])


    # setup is a method which generates the app menubar and buttons and initializes some vars
    def setup(self):
        # menubar structure     
        # |-file
        # |  |-open
        # |  |-next
        # |  |-import
        # |  |-save
        # |  |  |-picks
        # |  |  |-figure
        # |  |  |-processed data
        # |  |-settings
        # |  |  |-preferences
        # |  |  |-set working directory
        # |  |  |-set output directory
        # |  |-exit
        # |-pick
        # |  |-surface
        # |  |  |-new
        # |  |  |-end
        # |  |  |-clear
        # |  |-subsurface
        # |  |  |-new
        # |  |  |-end
        # |  |  |-clear
        # |  |  |-clear file
        # |-map
        # |  |-open
        # |-processing
        # |  |-dewow
        # |  |-remove mean trace
        # |  |-filter
        # |  |  |-low pass
        # |  |  |-high pass
        # |  |-gain
        # |  |  |-acg
        # |-help
        # |  |-instructions
        # |  |-keyboard shortcuts

        # generate menubar
        menubar = tk.Menu(self.parent)

        # create individual menubar items
        fileMenu = tk.Menu(menubar, tearoff=0)
        pickMenu = tk.Menu(menubar, tearoff=0)
        interpretMenu = tk.Menu(menubar, tearoff=0)
        procMenu = tk.Menu(menubar, tearoff=0)
        viewMenu = tk.Menu(menubar, tearoff=0)
        helpMenu = tk.Menu(menubar, tearoff=0)

        ### file menu items ###
        # open submenu
        openMenu = tk.Menu(fileMenu,tearoff=0)
        openMenu.add_command(label="Project", command=self.open_proj)
        openMenu.add_command(label="Data File    [Ctrl+O]", command=self.choose_dfile)
        openMenu.add_command(label="Basemap  [Ctrl+M]", command=self.init_bm)
        openMenu.add_command(label="Notepad", command=self.init_notepad)

        fileMenu.add_cascade(label="Open", menu=openMenu)
        fileMenu.add_command(label="Last      [◀]", command=lambda:self.switch_dfile(direction="Left"))
        fileMenu.add_command(label="Next      [▶]", command=lambda:self.switch_dfile(direction="Right"))

        # export submenu
        exportMenu = tk.Menu(fileMenu,tearoff=0)
        exportMenu.add_command(label="Project", command=self.export_proj)
        datExportMenu = tk.Menu(exportMenu,tearoff=0)
        datExportMenu.add_command(label="Raw Amplitude", command=lambda:self.export_dat(type="raw"))
        datExportMenu.add_command(label="Processed Amplitude", command=lambda:self.export_dat(type="proc"))
        exportMenu.add_cascade(label="Radar Data", menu=datExportMenu)
        pickExportMenu = tk.Menu(exportMenu,tearoff=0)
        pickExportMenu.add_command(label="Horizon", command=self.export_pick)
        pickExportMenu.add_command(label="All Horizons", command=lambda:self.export_pick(flag="all"))
        pickExportMenu.add_command(label="Merged Horizons  [Ctrl+S]", command=lambda:self.export_pick(flag="merged"))
        exportMenu.add_cascade(label="Picks", menu=pickExportMenu)
        exportMenu.add_command(label="Figure", command=self.export_fig)
        exportMenu.add_command(label="Processing Script", command=self.export_log)
        fileMenu.add_cascade(label="Export", menu=exportMenu)
        fileMenu.add_separator()

        # settings submenu
        settingsMenu = tk.Menu(fileMenu,tearoff=0)
        settingsMenu.add_command(label="Preferences", command=self.settings)
        settingsMenu.add_command(label="Set Working Folder", command=self.set_home)
        settingsMenu.add_command(label="Set Output Folder", command=self.set_out)

        fileMenu.add_cascade(label="Settings", menu=settingsMenu)
        fileMenu.add_separator()
    
        fileMenu.add_command(label="Exit  [Ctrl+Q]", command=self.close_window)

        ### interpret menu items ###
        pickMenu = tk.Menu(interpretMenu,tearoff=0)
        pickMenu.add_command(label="Start       [Ctrl+N]", command=self.start_pick)
        pickMenu.add_command(label="Stop        [Escape]", command=self.end_pick)
        interpretMenu.add_cascade(label="Pick", menu=pickMenu)
        newMenu = tk.Menu(interpretMenu,tearoff=0)
        newMenu.add_command(label="Horizon", command=self.new_horizon)
        newMenu.add_command(label="Segment", command=self.new_segment)
        interpretMenu.add_cascade(label="New", menu=newMenu)
        editMenu = tk.Menu(interpretMenu,tearoff=0)
        editMenu.add_command(label="Segment", command=self.edit_pick)
        interpretMenu.add_cascade(label="Edit", menu=editMenu)
        rmMenu = tk.Menu(interpretMenu,tearoff=0)
        rmMenu.add_command(label="Horizon", command=lambda:self.clear_pick(hFlag=True))
        rmMenu.add_command(label="Segment", command=lambda:self.clear_pick(segFlag=True))
        rmMenu.add_command(label="All", command=lambda:self.clear_pick(allFlag=True))
        rmMenu.add_command(label="From File", command=self.delete_datafilePicks)
        interpretMenu.add_cascade(label="Remove", menu=rmMenu)
        interpretMenu.add_command(label="Import", command=self.import_pick)
        srfMenu = tk.Menu(interpretMenu,tearoff=0)
        srfMenu.add_command(label="Define Horizon", command=self.srf_define)
        srfMenu.add_command(label="Auto-pick", command=self.srf_autopick)
        interpretMenu.add_cascade(label="Surface", menu=srfMenu)        
        exportMenu = tk.Menu(interpretMenu,tearoff=0)
        exportMenu.add_command(label="Horizon", command=self.export_pick)
        exportMenu.add_command(label="All Horizons", command=lambda:self.export_pick(flag="all"))
        exportMenu.add_command(label="Merged Horizons  [Ctrl+S]", command=lambda:self.export_pick(flag="merged"))
        interpretMenu.add_cascade(label="Export", menu=exportMenu)

        ### processing menu items ###
        procMenu.add_command(label="Reverse Radargram", command=lambda:self.procTools("reverse"))
        procMenu.add_command(label="Set Time Zero", command=lambda:self.procTools("tzero"))
        procMenu.add_command(label="Flatten", command=lambda:self.procTools("flatten"))
        procMenu.add_command(label="Restack", command=lambda:self.procTools("restack"))
        procMenu.add_command(label="Vertical Data Roll", command=lambda:self.procTools("vroll"))
        # procMenu.add_command(label="Dewow", command=lambda:self.procTools("dewow"))

        # processing submenu items
        gainMenu = tk.Menu(procMenu,tearoff=0)

        # filtering related items
        procMenu.add_command(label="Background Removal", command=lambda:self.procTools("remSlidingMean"))
        procMenu.add_command(label="Filter", command=lambda:self.procTools("filter"))
        procMenu.add_command(label="Hilbert Xform", command=lambda:self.procTools("hilbert"))

        # gain submenu items
        # gainMenu.add_command(label="AGC", command=lambda:self.procTools("agc"))
        gainMenu.add_command(label="T-Pow", command=lambda:self.procTools("tpow"))
        procMenu.add_cascade(label="Gain", menu=gainMenu)

        procMenu.add_command(label="Undo [Ctrl+Z]", command=lambda:self.procTools("undo"))        
        procMenu.add_command(label="Reset", command=lambda:self.procTools("reset"))

        ### view menu items ###
        viewMenu.add_checkbutton(label="Interpretations", onvalue=True, offvalue=False, variable=self.pick_vis, command=self.set_pick_vis)
        viewMenu.add_checkbutton(label="Labels", onvalue=True, offvalue=False, variable=self.ann_vis, command=self.set_ann_vis)

        ### help menu items ###
        helpMenu.add_command(label="Instructions", command=self.help)
        helpMenu.add_command(label="Keyboard Shortcuts", command=self.shortcuts)

        # add items to menubar
        menubar.add_cascade(label="File", menu=fileMenu)
        menubar.add_cascade(label="Interpretation", menu=interpretMenu)
        menubar.add_cascade(label="Processing", menu=procMenu)
        menubar.add_cascade(label="View", menu=viewMenu)
        menubar.add_cascade(label="Help", menu=helpMenu)
        
        # add the menubar to the window
        self.parent.config(menu=menubar)

        # configure impick and wvpick tabs
        self.nb = ttk.Notebook(self.parent)
        self.nb.pack(side="top",anchor='w', fill="both", expand=1)
        self.imTab = tk.Frame(self.parent)
        self.imTab.pack()
        self.nb.add(self.imTab, text='Profile')
        self.wvTab = tk.Frame(self.parent)
        self.wvTab.pack()
        self.nb.add(self.wvTab, text='Waveform')

        # bind tab change event to send pick data to wvpick if tab switched from impick
        self.nb.bind("<<NotebookTabChanged>>", self.tab_change)

        # check cmap
        self.check_cmap()

        # initialize impick
        self.impick = impick.impick(self.imTab, button_tip, self.popup, self.figsettings["fontsize"].get())
        self.impick.set_vars()
        self.impick.update_figsettings(self.figsettings)
        self.impick.set_eps_r(self.eps_r.get())

        # initialize wvpick
        self.wvpick = wvpick.wvpick(self.wvTab, button_tip, self.reset_wvpick)
        self.wvpick.update_figsettings(self.figsettings)
        self.wvpick.set_vars()

        # initialize note
        self.notepad = notepad.notepad(parent=self.parent, init_dir=self.conf["path"]["outPath"])

        # set up  info frame
        infoFrame = tk.Frame(self.parent)
        infoFrame.pack(side="bottom",fill="x")
        self.rinfolbl = tk.Label(infoFrame)
        self.rinfolbl.pack(side="left")

        # handle x-button closing of window
        self.parent.protocol("WM_DELETE_WINDOW", self.close_window)

        # bind keypress events
        self.parent.bind("<Key>", self.key)


    # key is a method to handle UI keypress events
    def key(self,event):
        state = event.state
        key = event.keysym
        # event.state & 4 True for Ctrl+Key
        # event.state & 1 True for Shift+Key
        # general keps for either tab
        # Ctrl+O open file
        if state & 4 and key == "o":
            self.choose_dfile()

        # Ctrl+S save picks
        elif state & 4 and key == "s":
            self.export_pick(flag="merged")

        # Ctrl+M open map
        elif state & 4 and key == "m":
            self.init_bm()

        # Ctrl+Q close RAGU
        elif state & 4 and key == "q":
            self.close_window()

        # profile view keys
        if self.tab == "Profile":
            # Space key to toggle impick between radar image and sim
            if key =="space":
                self.impick.set_im(from_gui=True)

            elif state & 4 and key == "n":
                self.start_pick()

            # Escape key to stop picking current layer
            elif key == "Escape":
                self.end_pick()

            # BackSpace to clear last pick 
            elif key =="BackSpace":
                self.impick.clear_last()

            # c key to clear all picks in impick
            elif key =="c":
                # clear picks
                self.clear_pick(allFlag=True)

            # right key next file
            elif key =="Left" or key == "Right":
                self.switch_dfile(direction=key)

            # h key to set axes limits to home extent
            elif key =="h":
                self.impick.fullExtent()

            # plus key to zoom in
            elif key =="plus":
                self.impick.zoomIn()

            # minus key to zoom out
            elif key =="minus":
                self.impick.zoomOut()

            # d key to set axes limits to pan right
            elif key =="d":
                self.impick.panRight()

            # a key to set axes limits to pan left
            elif key =="a":
                self.impick.panLeft()

            # w key to set axes limits to pan up
            elif key =="w":
                self.impick.panUp()

            # s key to set axes limits to pan down
            elif  key=="s":
                self.impick.panDown()

            # Ctrl+z undo last processing
            elif state & 4 and key == "f":
                            self.procTools(arg="filter")

            # Ctrl+z undo last processing
            elif state & 4 and key == "z":
                            self.procTools(arg="undo")

            # Ctrl+z undo last processing
            elif state & 4 and key == "y":
                            self.procTools(arg="redo")

        # waveform view keys
        if self.tab == "Waveform":
            # h key to set axes limits to home extent
            if key =="h":
                self.wvpick.fullExtent()
    
            # a key to step back in trace count
            elif key =="Right":
                self.wvpick.stepForward()

            # d key to step forward in trace count
            elif key =="Left":
                self.wvpick.stepBackward()


    # save_check is a method to check if picks have been saved - a bit messy (need to account for potential "srf" ingested upon data load)
    def save_check(self):
        if self.rdata:
            # if horizons exist, but no output pick df
            if (self.rdata.pick.get_pick_flag()) and (self.rdata.out is None):
                horizons = list(self.rdata.pick.horizons.keys())
                # if srf is only horizon
                if "srf" in horizons and len(horizons) == 1:
                    # if srf was not ingested upon data load rdata.srfElev will be all nan
                    if np.isnan(self.rdata.srfElev).all():
                        return False
                    else:
                        return True
                # if non-srf horizons exist
                else:
                    return False
            # if no horizons exist or horizons already exported
            else:
                return True


    # close_window is a gui method to exit RAGU
    def close_window(self):
        # check if picks have been made and saved
        if self.save_check() == False:
            if tk.messagebox.askokcancel("Warning", "Exit RAGU without saving picks?", icon = "warning") == True:
                self.parent.destroy()
        else:
            self.parent.destroy()


    # set_home is a method to set the session home directory
    def set_home(self):
        self.datPath = tk.filedialog.askdirectory(title="root data directory",
                                       initialdir=self.datPath,
                                       mustexist=True)    
    

    # set_out is a method to set the session output directory
    def set_out(self):
        self.conf["path"]["outPath"] = tk.filedialog.askdirectory(title="output data directory",
                                       initialdir=self.conf["path"]["outPath"],
                                       mustexist=True)


    # open project
    def open_proj(self, proj_loadName=None):
        # save_check
        if (self.save_check() == False) and (tk.messagebox.askyesno("Warning", "Discard unsaved picks?", icon = "warning") == False):
            return

        # # if h5 file already open, save pick layer to data file
        # if (self.rdata) and (self.rdata.out is None) and (self.rdata.fpath.endswith(".h5")) and (self.rdata.dtype=="oibak"):
        #     export.h5(self.rdata.fpath, pd.DataFrame({"bed_twtt":np.repeat(np.nan, self.rdata.tnum)}), self.rdata.dtype)

        if proj_loadName is None:
            # select input file
            if self.os == "darwin":
                proj_loadName = tk.filedialog.askopenfilename(initialdir = self.conf["path"]["outPath"] + '/../', title = "select project file")
            else:
                proj_loadName = tk.filedialog.askopenfilename(initialdir = self.conf["path"]["outPath"] + '/../',title = "select project file",filetypes = [("ragu project", ".ragu"),
                                                                                                                            ("all files",".*")])
        
        if proj_loadName:
            try:
                self.proj.set_projPath(proj_loadName)
                self.proj.load()
                if self.proj.get_datPath():
                    self.open_dfile(self.proj.get_datPath())
                if self.proj.get_mapPath():
                    self.init_bm(self.proj.get_mapPath())
                if self.proj.get_notePath():
                    self.init_notepad(self.proj.get_notePath())
            except Exception as err:
                print("Load project error: {}".format(err))


    # choose_dfile is a gui method which has the user select and input data file - then passed to impick.load()
    def choose_dfile(self):
        # save_check
        if (self.save_check() == False) and (tk.messagebox.askyesno("Warning", "Discard unsaved picks?", icon = "warning") == False):
            return

        # # if h5 file already open, save pick layer to data file
        # if (self.rdata) and (self.rdata.out is None) and (self.rdata.fpath.endswith(".h5")) and (self.rdata.dtype=="oibak"):
        #     export.h5(self.rdata.fpath, pd.DataFrame({"bed_twtt":np.repeat(np.nan, self.rdata.tnum)}), self.rdata.dtype)

        # select input file
        if self.os == "darwin":
            temp_loadName = tk.filedialog.askopenfilename(initialdir = self.datPath,title = "select data file")
        else:
            temp_loadName = tk.filedialog.askopenfilename(initialdir = self.datPath,title = "select data file",filetypes = [("all files",".*"),
                                                                                                                            ("hd5f", ".mat .h5"),
                                                                                                                            ("sharad", ".img"),
                                                                                                                            ("marsis", ".dat"),
                                                                                                                            ("pulseekko", ".DT1"),
                                                                                                                            ("gssi",".DZT")])

        if temp_loadName.endswith(".ragu"):
            self.open_proj(proj_loadName=temp_loadName)
        
        else:
            self.open_dfile(temp_loadName)


    # open_dat loads the data file and passes to other modules
    def open_dfile(self, f_loadName=None, switch=False, direction="Right"):
            # if input selected, clear impick canvas, ingest data and pass to impick
            try:
            # for fuck in ['bananas']:
                if f_loadName:
                    # switch to profile tab
                    if self.tab == "Waveform":
                        self.nb.select(self.nb.tabs()[0])
                    self.f_loadName = f_loadName
                    # update and save project file
                    self.proj.update_paths(self.f_loadName, self.map_loadName, self.notepad._notepad__get_file())
                    self.proj.save()
                    # ingest the data
                    self.igst = ingest(self.f_loadName)
                    self.rdata = self.igst.read(self.conf["path"]["simPath"], self.conf["nav"]["crs"], self.conf["nav"]["body"])
                    try:
                        self.rdata.asep =  float(self.conf["output"]["asep"])
                        self.rdata.info["Antenna Separation [m]"] = self.rdata.asep
                    except:
                        pass
                    self.impick.clear_canvas()  
                    self.impick.set_vars()
                    self.impick.load(self.rdata)
                    # set existing horizons
                    for horizon in self.rdata.pick.horizons.keys():
                        self.impick.set_picks(horizon=horizon)
                    self.impick.set_pickState(state=False)
                    self.impick.update_hor_opt_menu()
                    self.impick.update_seg_opt_menu
                    self.impick.set_axes()
                    self.impick.drawData()
                    self.impick.update_pickLabels()
                    self.impick.update_bg()
                    self.wvpick.set_vars()
                    self.wvpick.clear()
                    self.wvpick.set_data(self.rdata)
                    self.rinfolbl.config(text = '\t\t'.join('{}: {}'.format(k, d) for k, d in self.rdata.info.items()))

                # pass basemap to impick for plotting pick location
                if self.map_loadName and self.basemap.get_state() == 1:
                    self.basemap.set_track(self.rdata.fn)
                    self.basemap.set_nav(self.rdata.fn, self.rdata.navdf)
                    self.basemap.plot_tracks()
                    self.impick.get_basemap(self.basemap)
        
                if self.rdata and self.notepad._notepad__get_state() == 1:
                    self.notepad._notepad__write_track(fn=self.rdata.fn)
                    if self.notepad._notepad__get_file():
                        self.notepad._notepad__saveFile()

                # see if user would like  to load previous pick file
                tmpf = self.conf["path"]["outPath"] + self.rdata.fn + "_pk_" + self.conf["param"]["uid"] + ".csv"
                if (os.path.isfile(tmpf)) and (tk.messagebox.askyesno("Load Picks", "Load pick file: {}?".format(tmpf), icon = "question") == True):
                    self.import_pick(tmpf)


            # recall choose_dfile if wrong file type is selected 
            except Exception as err:
                # if switch, just advance to next file
                if switch:
                    self.switch_dfile(direction)
                else:
                    print(err)
                    self.choose_dfile() 


    # switch_dfile is a method to get the filename of the last/next data file in the directory to open
    def switch_dfile(self, direction="Right"):
        if self.tab == "Profile" and self.f_loadName:
            # prompt save check
            if (self.save_check() == False) and (tk.messagebox.askyesno("Warning", "Discard unsaved picks?", icon = "warning") == False):
                return

            # # if h5 file already open, save pick layer to data file
            # if (self.rdata) and (self.rdata.out is None) and (self.rdata.fpath.endswith(".h5")) and (self.rdata.dtype=="oibak"):
            #     export.h5(self.rdata.fpath, pd.DataFrame({"bed_twtt":np.repeat(np.nan, self.rdata.tnum)}), self.rdata.dtype)
            
            file_path = os.path.dirname(self.f_loadName)

            # get index of crurrently displayed file in directory
            file_list = [file_path + "/" + f for f in sorted(os.listdir(file_path)) if f.endswith(self.igst.ftype) or f.endswith(self.igst.ftype.upper())]
            file_index = file_list.index(self.f_loadName)

            if direction=="Right":
                # add one to index to load next file
                file_index += 1

                # check if more files exist in directory following current file
                if file_index <= (len(file_list) - 1):
                    self.open_dfile(file_list[file_index], True, direction)

                else:
                    print("Note: " + self.f_loadName.split("/")[-1] + " is the last file in " + file_path + "/*." + self.f_loadName.split(".")[-1])

            elif direction=="Left":
                # add one to index to load next file
                file_index -= 1

                # check if more files exist in directory following current file
                if file_index >= 0:
                    self.open_dfile(file_list[file_index], True, direction)

                else:
                    print("Note: " + self.f_loadName.split("/")[-1] + " is the first file in " + file_path + "/*." + self.f_loadName.split(".")[-1])
    

    # generate new interpretation horizon
    def new_horizon(self):
        if self.f_loadName:
            self.impick.init_horizon()


    # generate new interpretation segment for specified horizon
    def new_segment(self):
        if self.f_loadName:
            self.impick.init_segment()


    # start impick picking functionality
    def start_pick(self):
        if self.f_loadName:
            self.impick.set_pickState(True)


    # end impick picking functionality
    def end_pick(self):
        if self.f_loadName:
            self.impick.set_pickState(False)


    def edit_pick(self):
        if self.f_loadName:
            self.impick.edit_segment()


    def clear_pick(self, hFlag=None, segFlag=None, allFlag=None):
        if self.f_loadName:
            if hFlag:
                self.impick.rm_horizon()
            if segFlag:
                self.impick.rm_segment()
            if allFlag:
                self.impick.rm_horizon(rm_all=True)

    
    # import_pick is a method to load and plot picks saved to a csv file
    def import_pick(self, path=None):
        if self.f_loadName:
            if path:
                if os.path.isfile(path):
                    pk_file = path
                else:
                    pk_file = ""
            else:
                pk_file = ""
                pk_file = tk.filedialog.askopenfilename(initialdir = self.conf["path"]["outPath"], title = "load picks", filetypes = (("comma separated value", "*.csv"),))
            if pk_file:
                horizons = self.igst.import_pick(pk_file, self.conf["param"]["uid"])
                for horizon in horizons:
                    self.impick.set_picks(horizon=horizon)
                self.impick.blit()


    # srf_define
    def srf_define(self, srf=None):
        horizons = list(self.rdata.pick.horizons)
        for h in ["srf","surf","surface"]:
            if h in horizons:
                srf = h
                break

        if (srf is None) and (len(horizons) > 0):
            if self.popup.flag == 1:
                return
            horizon_colors = self.impick.get_horizon_colors()
            # create popup window to get new horizon name and interpreatation color
            hname =tk.StringVar()
            hname.set(horizons[-1])
            popup = self.popup.new(title="Surface Horizon")
            tk.Label(popup, text="Select surface horizon from which to reference subsurface interpretations:").pack(fill="both", expand=True)
            dropdown = tk.OptionMenu(popup, hname, *horizons)
            dropdown["menu"].delete(0, "end")
            dropdown.config(width=20)
            dropdown.pack(fill="none", expand=True)
            self.set_menu_color(menu=dropdown, horizon=hname, colors=horizon_colors)
            # trace change in self.color to self.set_menu_color
            trace = hname.trace("w", lambda *args, menu=dropdown, horizon=hname, colors=horizon_colors : self.set_menu_color(menu, horizon, colors))
            for key, val in horizon_colors.items():
                dropdown["menu"].add_command(label=key, foreground=val, activeforeground=val, command=tk._setit(hname, key))
            button = tk.Button(popup, text="OK", command=lambda:self.popup.close(flag=0), width=20).pack(side="left", fill="none", expand=True)
            button = tk.Button(popup, text="Cancel", command=lambda:[hname.set(""), self.popup.close(flag=-1)], width=20).pack(side="left", fill="none", expand=True)
            # wait for window to be closed
            self.parent.wait_window(popup)
            # remove the trace
            hname.trace_vdelete("w", trace)
            srf = hname.get()
            if (not srf) or (self.popup.flag == -1):
                return
        self.rdata.pick.set_srf(srf)
        self.rdata.set_srfElev()


    # srf_autopick
    def srf_autopick(self):
        srf = self.rdata.pick.get_srf()
        if srf is None:
            srf = "srf" 
        elif not tk.messagebox.askyesno("Warning","Surface horizon already exists. Overwrite with auto-pick surface?"):
            return

        self.rdata.pick.horizons[srf] = utils.get_srf(np.abs(self.rdata.dat), self.rdata.info["Signal Type"])
        self.srf_define(srf=srf)
        self.impick.set_picks(horizon=srf)
        self.impick.blit()


    # export_proj
    def export_proj(self):
        # select input file
        tmp = self.proj.get_projPath()
        if not tmp:
            if self.os == "darwin":
                tmp = tk.filedialog.asksaveasfilename(initialdir = self.conf["path"]["outPath"], title = "save project file")
            else:
                tmp = tk.filedialog.asksaveasfilename(initialdir = self.conf["path"]["outPath"], title = "save project file",filetypes = [("ragu project", ".ragu"),
                                                                                                                            ("all files",".*")])
        
        if tmp:
            fn, ext = os.path.splitext(tmp)

            try:
                self.proj.set_projPath(fn + ".ragu")
                self.proj.update_paths(self.f_loadName, self.map_loadName, self.notepad._notepad__get_file())
                self.proj.save()
                print("Project file saved successfully:\t{}".format(fn + ".ragu"))

            except Exception as err:
                print("Save project error: {}".format(err))

        return

    # export_pick is method to receieve the desired pick save location from user input
    def export_pick(self, flag=None):
        if self.f_loadName:
            # see if any picks have been made
            if  self.rdata.pick.get_pick_flag():
                # get directory path to save data
                dir_out = tk.filedialog.askdirectory(title = "Save Location",
                                                    initialdir = self.conf["path"]["outPath"],
                                                    mustexist = True)

                if not dir_out:
                    return

                # end any active picking
                self.end_pick()
                # sort horizons by mean twtt in array
                self.rdata.pick.horizons = utils.sort_array_dict(self.rdata.pick.horizons, self.rdata.pick.get_srf())
                # get horizon to export
                horizon = None
                horizons = list(self.rdata.pick.horizons)

                # if flag is none export single horizon
                if flag is None:
                    if self.popup.flag == 1:
                        return
                    horizon_colors = self.impick.get_horizon_colors()
                    # create popup window to get new horizon name and interpreatation color
                    hname =tk.StringVar()
                    hname.set(horizons[-1])
                    popup = self.popup.new(title="Export Horizon")
                    tk.Label(popup, text="Select horizon to export:").pack(fill="both", expand=True)
                    dropdown = tk.OptionMenu(popup, hname, *horizons)
                    dropdown["menu"].delete(0, "end")
                    dropdown.config(width=20)
                    dropdown.pack(fill="none", expand=True)
                    self.set_menu_color(menu=dropdown, horizon=hname, colors=horizon_colors)
                    # trace change in self.color to self.set_menu_color
                    trace = hname.trace("w", lambda *args, menu=dropdown, horizon=hname, colors=horizon_colors : self.set_menu_color(menu, horizon, colors))
                    for key, val in horizon_colors.items():
                        dropdown["menu"].add_command(label=key, foreground=val, activeforeground=val, command=tk._setit(hname, key))
                    button = tk.Button(popup, text="OK", command=lambda:self.popup.close(flag=0), width=20).pack(side="left", fill="none", expand=True)
                    button = tk.Button(popup, text="Cancel", command=lambda:[hname.set(""), self.popup.close(flag=-1)], width=20).pack(side="left", fill="none", expand=True)
                    # wait for window to be closed
                    self.parent.wait_window(popup)
                    # remove the trace
                    hname.trace_vdelete("w", trace)
                    horizon = hname.get()
                    if (not horizon) or (self.popup.flag == -1):
                        return
                    else:
                        fn_out = dir_out + "/" + self.rdata.fn + "_" + horizon + "_pk_" + self.conf["param"]["uid"]


                # elif flag is merged export, append outfile name with _pk_uid
                elif flag=="merged":
                    fn_out = dir_out + "/" + self.rdata.fn + "_pk_" + self.conf["param"]["uid"]
                    if self.rdata.pick.get_srf() is None:
                        self.srf_define()
                    # update srf_elev
                    if self.rdata.pick.get_srf():
                        self.rdata.set_srfElev()
    
                # export single or merged horizons
                if flag is None or flag=="merged":
                    # ensure surface horizon is defined
                    self.srf_define()
                    # set output dataframe
                    self.rdata.set_out(export.pick_math(self.rdata, self.eps_r.get(), self.conf["output"]["amp"], horizon=horizon, srf=self.rdata.pick.get_srf()))
                    if self.conf["output"].getboolean("csv"):
                        export.csv(fn_out + ".csv", self.rdata.out)
                    if self.conf["output"].getboolean("gpkg"):
                        export.gpkg(fn_out + ".gpkg", self.rdata.out, self.conf["nav"]["crs"])


                # if flag is all, export all horizons individually
                elif flag == "all":
                    for h in horizons:
                        fn_out = dir_out + "/" + self.rdata.fn + "_" + h + "_pk_" + self.conf["param"]["uid"]
                        # export picks for each horizon
                        self.rdata.set_out(export.pick_math(self.rdata, self.eps_r.get(), self.conf["output"]["amp"], horizon=h, srf=self.rdata.pick.get_srf()))
                        if self.conf["output"].getboolean("csv"):
                            export.csv(fn_out + ".csv", self.rdata.out)
                        if self.conf["output"].getboolean("gpkg"):
                            export.gpkg(fn_out + ".gpkg", self.rdata.out, self.conf["nav"]["crs"])
                        fn_out = dir_out + "/" + self.rdata.fn + "_pk_" + self.conf["param"]["uid"]
                        

                if self.conf["output"].getboolean("fig"):
                    self.impick.export_fig(fn_out + ".png")
                # export project if projpath exists
                if self.proj.get_projPath():
                    self.export_proj()


    # export_dat is a method to save processed radar data
    def export_dat(self, type="raw"):
        if self.f_loadName:
            tmp_fn_out = ""
            if self.os == "darwin":
                tmp_fn_out = tk.filedialog.asksaveasfilename(initialfile = os.path.splitext(self.f_loadName.split("/")[-1])[0] + "_" + type + "_amp.csv",
                                initialdir = self.conf["path"]["outPath"], title = "save processed data")
            else:
                tmp_fn_out = tk.filedialog.asksaveasfilename(initialfile = os.path.splitext(self.f_loadName.split("/")[-1])[0] + "_" + type + "_amp",
                                initialdir = self.conf["path"]["outPath"], title = "save processed data", filetypes = [("comma-separated values",".csv")])
            if tmp_fn_out:
                fn, ext = os.path.splitext(tmp_fn_out)

                if type == "raw":
                    export.dat(fn + ".csv", self.rdata.get_dat())
                elif type == "proc":
                    export.dat(fn + ".csv", self.rdata.proc.get_curr_amp())


    # export_log is a method to save the processing log
    def export_log(self):
        if self.f_loadName:
            tmp_fn_out = ""
            tmp_fn_out = tk.filedialog.asksaveasfilename(initialfile = os.path.splitext(self.f_loadName.split("/")[-1])[0] + ".py",
                            initialdir = self.conf["path"]["outPath"], title = "save processing script")

            if tmp_fn_out:
                fn, ext = os.path.splitext(tmp_fn_out)

                export.log(fn + ".py", self.rdata.hist)

    # export_fig is a method to export the radargram image
    def export_fig(self):
        if self.f_loadName:
            tmp_fn_out = ""
            if self.os == "darwin":
                tmp_fn_out = tk.filedialog.asksaveasfilename(initialfile = os.path.splitext(self.f_loadName.split("/")[-1])[0] + ".png",
                                initialdir = self.conf["path"]["outPath"], title = "save radar image")
            else:
                tmp_fn_out = tk.filedialog.asksaveasfilename(initialfile = os.path.splitext(self.f_loadName.split("/")[-1])[0],
                                initialdir = self.conf["path"]["outPath"], title = "save radar image", filetypes = [("png image", ".png")])
            if tmp_fn_out:
                fn, ext = os.path.splitext(tmp_fn_out)

            self.impick.export_fig(fn + ".png")


    # init_notepad is a method to initialize the ragu notepad widget
    def init_notepad(self, path=""):
        if self.notepad._notepad__get_state() == 0:
            self.notepad._notepad__setup(path=path)
            if path:
                self.notepad._notepad__openFile()
            if self.rdata:
                self.notepad._notepad__write_track(self.rdata.fn)


    # init_bm is a method to get the desired basemap location and initialize
    def init_bm(self, path=None):
        if not path:
            path = ""
            if self.os == "darwin":
                path = tk.filedialog.askopenfilename(initialdir = self.conf["path"]["mapPath"], title = "select basemap file")

            else:
                path = tk.filedialog.askopenfilename(initialdir = self.conf["path"]["mapPath"], title = "select basemap file", filetypes = [("geotiff files","*.tif"),
                                                                                                                                                        ("all files","*.*")])
        if path:
            # initialize basemap if not currently open
            if not self.map_loadName or self.basemap.get_state() == 0:
                self.basemap = basemap.basemap(self.parent, self.datPath, self.conf["nav"]["crs"], self.conf["nav"]["body"], self.from_basemap)
            self.map_loadName = path
            self.proj.set_mapPath(self.map_loadName)
            self.basemap.set_vars()
            self.basemap.map(self.map_loadName)

            if self.f_loadName:
                # pass basemap to impick for plotting pick location
                self.basemap.clear_nav()
                self.basemap.set_track(self.rdata.fn)
                self.basemap.set_nav(self.rdata.fn, self.rdata.navdf)
                self.basemap.plot_tracks()
                self.impick.get_basemap(self.basemap)


    # return selected track from basemap frame
    def from_basemap(self, path, track):
        # find matching file to pass to open_loc - ensure valid ftype
        f = [_i for _i in os.listdir(path) if track in _i]
        for _i in f:
            try:
                ingest(_i.split(".")[-1])
                break
            except Exception:
                continue

        # pass file to open_data
        self.open_dfile(path + _i)


    # reset_wvpick passes picks from impick to wvpick
    def reset_wvpick(self, force=False, check=False):
        if self.wvpick.get_horizon_paths() is None:
            force = True
        else:
            impaths = self.impick.get_horizon_paths()
            wvpaths = self.wvpick.get_horizon_paths()
            for h in impaths.keys():
                if h not in wvpaths.keys():
                    check = True
                    break
                for s in impaths[h].keys():
                    if s not in wvpaths[h].keys():
                        check = True
                        break
            if check and tk.messagebox.askyesno("Import","Import updated horizon interpretations?"):
                force = True
        if force:
            self.wvpick.set_data(self.rdata)
            self.wvpick.set_horizon_colors(self.impick.get_horizon_colors())
            self.wvpick.set_horizon_paths(self.impick.get_horizon_paths())
            self.wvpick.set_picks()
            self.wvpick.plot_wv()


    # set tkinter menu font colors to match color name
    def set_menu_color(self, menu=None, horizon=None, colors=None, *args):
        if not [x for x in (menu, colors) if x is None]:
            h = horizon.get()
            if h:
                c = colors[h]
                menu.config(foreground=c, activeforeground=c, highlightcolor=c)


    # change tabs between profile and waveform views
    def tab_change(self, event):
        selection = event.widget.select()
        tmp = event.widget.tab(selection, "text")
        # don't do anything unless different tab selected
        if (tmp != self.tab) and (self.rdata):
            self.tab = tmp
            # determine which tab is active
            if (self.tab == "Waveform"):
                self.end_pick()
                # reset_wavepick
                self.reset_wvpick()
            elif (self.tab == "Profile"):
                # get updated picks from wvpick and pass back to impick if they differ
                if not (utils.compare_horizon_paths(self.wvpick.get_horizon_paths(), self.impick.get_horizon_paths())) and \
                        (tk.messagebox.askyesno("Import","Import optimized horizon interpretations?")):
                    self.impick.set_horizon_paths(self.wvpick.get_horizon_paths())
                    self.impick.blit()


    # delete_datafilePicks is a method to clear subsurface picks saved to the data file
    def delete_datafilePicks(self):
            if self.f_loadName and tk.messagebox.askokcancel("Warning", "Delete any existing data file bed picks?", icon = "warning") == True:
                utils.delete_savedPicks(self.f_loadName)


    # processing tools
    def procTools(self, arg = None):
        if self.f_loadName:
            procFlag = None
            simFlag = None
            if arg == "reverse":
                self.rdata.reverse()
                self.impick.reverse()
                procFlag = True

            elif arg == "tzero":
                # set tzero should only be used for ground-based GPR data
                self.rdata.set_tzero()
                if self.rdata.flags.sampzero > 0:
                    srf = self.rdata.pick.get_srf()
                    if srf is None:
                        srf = "srf"

                    # define surface horizon name to set index to zeros
                    self.rdata.pick.horizons[srf] = np.zeros(self.rdata.tnum)
                    self.srf_define(srf=srf)

                    # apply sample shift to existing horizons and update picks
                    for h in self.rdata.pick.horizons:
                        if h != srf:
                            self.rdata.pick.horizons[h] -= self.rdata.flags.sampzero
                    
                    # clear horizons from canvas and redraw shifted horizons
                    tmp = copy.deepcopy(self.rdata.pick.horizons)
                    self.impick.rm_horizon(rm_all=True, verify=False)
                    for h in tmp.keys():
                        self.rdata.pick.horizons[h] = tmp[h]
                        self.impick.set_picks(h)

                    self.impick.blit()
                    procFlag = True

            elif arg == "flatten":
                # if a surface is defined and data isn't already time-zero shifted, flatten

                if (self.rdata.pick.get_srf()) and (self.rdata.flags.sampzero == 0):
                    self.rdata.flatten()

                    # apply sample shift to existing horizons and update picks
                    # first copy horizons and remove them from impick cancas
                    tmp = copy.deepcopy(self.rdata.pick.horizons)
                    self.impick.rm_horizon(rm_all=True, verify=False)
                    for h in tmp:
                        self.rdata.pick.horizons[h] = tmp[h] - self.rdata.flags.sampzero
                        self.impick.set_picks(horizon=h)

                    self.impick.blit()
                    procFlag = True

            elif arg == "vroll":
                samples = tk.simpledialog.askinteger("input","number of samples to roll data array")
                self.rdata.vertical_roll(samples)
                procFlag = True

            elif arg == "restack":
                thold = tk.simpledialog.askfloat("input","GPS drift threshold between consecutive traces (m)", initialvalue=0.5)
                dist = tk.simpledialog.askfloat("input","restacking distance (m)", initialvalue=0)
                self.rdata.restack(dist, thold)
                procFlag = True

            elif arg == "dewow":
                print("dewow currently in development")
                # window = tk.simpledialog.askfloat("input","dewow window size (# samples/" +  str(int(self.rdata.snum)) + ")?")
                # proc = processing.dewow(self.rdata.dat, window=10)
                # procFlag = True

            elif arg == "hilbert":
                print("hilbert transform currently in development")
                # self.rdata.hilbertxform()
                # procFlag = True

            elif arg == "filter":
                if self.popup.flag == 1:
                    return
                # create popup window to get filter cutoff and direction
                popup = self.popup.new(title="Butterworth Filter")

                def cutoffentry(*args):
                    if btype.get() == "lowpass":
                        lowcut_ent.config(state="disabled")
                        highcut_ent.config(state="normal")
                    elif btype.get() == "highpass":
                        lowcut_ent.config(state="normal")
                        highcut_ent.config(state="disabled")
                    elif btype.get() == "bandpass":
                        lowcut_ent.config(state="normal")
                        highcut_ent.config(state="normal")

                row = tk.Frame(popup)
                row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
                tk.Label(row, text="Filter Type: ").pack(side="left")
                btype = tk.StringVar()
                radio = tk.Radiobutton(row,text="Lowpass", variable=btype, value="lowpass")
                radio.pack(side="left")
                button_tip(self.parent, radio, "Pass all frequencies below the highcut frequency")

                radio = tk.Radiobutton(row,text="Highpass", variable=btype, value="highpass")
                radio.pack(side="left")
                button_tip(self.parent, radio, "Pass all frequencies above the lowcut frequency")

                radio = tk.Radiobutton(row,text="Bandpass", variable=btype, value="bandpass")
                radio.pack(side="left")
                button_tip(self.parent, radio, "Pass all frequencies above the lowcut frequency and below the highcut frequency")

                row = tk.Frame(popup)
                row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
                tk.Label(row, text="Filter Direction: ").pack(side="left")
                direction = tk.IntVar(value=0)
                radio = tk.Radiobutton(row,text="Fast-Time", variable=direction, value=0)
                radio.pack(side="left")
                button_tip(self.parent, radio, "Filter in the fast-time direction (along-trace)")

                radio = tk.Radiobutton(row,text="Slow-Time", variable=direction, value=1)
                radio.pack(side="left")
                button_tip(self.parent, radio, "Filter in the slow-time direction (along-track)")
            
                row = tk.Frame(popup)
                row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
                tk.Label(row, text="Lowcut Frequency: ").pack(side="left")
                lowcut = tk.StringVar()
                lowcut_ent = tk.Entry(row, textvariable=lowcut, width = 10)
                lowcut_ent.pack(side="left")
                lowcut_ent.delete(0, 'end')
                lowcut_ent.config(state="disabled")
                button_tip(self.parent, lowcut_ent, "Lower frequency bound")
                tk.Label(row, text="\tHighcut Frequency: ").pack(side="left")
                highcut = tk.StringVar()
                highcut_ent = tk.Entry(row, textvariable=highcut, width = 10)
                highcut_ent.pack(side="left")
                highcut_ent.delete(0, 'end')
                button_tip(self.parent, highcut_ent, "Upper frequency bound")

                row = tk.Frame(popup)
                row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
                tk.Label(row, text="Order: ").pack(side="left")
                order = tk.IntVar(value=5)
                ent = tk.Entry(row, textvariable=order, width = 10)
                ent.pack(side="left")
                button_tip(self.parent, ent, "Butterworth filter order")

                row = tk.Frame(popup)
                row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
                tk.Button(row, text="OK", command=lambda:self.popup.close(flag=0), width=10).pack(side="left", fill="none", expand=True)
                tk.Button(row, text="Cancel", command=lambda:self.popup.close(flag=-1), width=10).pack(side="left", fill="none", expand=True)

                btype.trace("w",cutoffentry)
                btype.set("lowpass")

                # wait for window to be closed
                self.parent.wait_window(popup)
                if self.popup.flag == -1:
                    return
                try:
                    if lowcut.get():
                        lowcut = float(lowcut.get())
                    else:
                        lowcut = None
                    if highcut.get():
                        highcut = float(highcut.get())
                    else:
                        highcut = None
                    if lowcut is None and highcut is None:
                        raise ValueError("Filter error: no lowcut or highcut frequencies specified.")
                    self.rdata.filter(btype=btype.get(), lowcut=lowcut, highcut=highcut, order=order.get(), direction=direction.get())
                    procFlag = True
                except Exception as err:
                    print(err)

            elif arg == "tpow":
                power = tk.simpledialog.askfloat("Input","Power for tpow gain?")
                self.rdata.tpowGain(power=power)
                procFlag = True

            elif arg == "remSlidingMean":
                window = tk.simpledialog.askinteger("Input","Window size for background removal (Number of Samples)?")
                if window <= self.rdata.tnum:
                    self.rdata.removeSlidingMeanFFT(window=window)
                    procFlag = True
                else:
                    raise raguError("Window size must not exceed the number of traces.")


            elif arg == "agc":
                print("automatic gain control currently in development")
                # window = tk.simpledialog.askfloat("input","AGC gain window size (# samples/" +  str(int(self.rdata.snum)) + ")?")
                # proc = processing.agcGain(self.rdata.dat, window=window)
                # procFlag = True

            elif arg == "undo":
                self.rdata.undo()
                procFlag = True

            elif arg == "redo":
                self.rdata.redo()
                procFlag = True

            elif arg == "reset":
                # reset origianl rdata
                self.rdata.reset()
                procFlag = True

            else:
                print("undefined processing method")
                exit(1)

            if procFlag:
                self.impick.set_crange()
                self.impick.drawData(force=True)
                self.wvpick.clear()
                self.wvpick.set_vars()
                self.wvpick.set_data(self.rdata)


    def settings(self):
        settingsWindow = tk.Toplevel(self.parent)

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        label = tk.Label(row, text = "General")
        label.pack(side=tk.TOP)
        f = tk.font.Font(label, label.cget("font"))
        f.configure(underline=True)
        label.configure(font=f)

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=25, text="Dielectric constant", anchor='w')
        lab.pack(side=tk.LEFT)
        self.epsEnt = tk.Entry(row,textvariable=self.eps_r)
        self.epsEnt.pack(side=tk.RIGHT, expand=tk.YES, fill=tk.X)

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=25, text="Debug mode", anchor='w')
        lab.pack(side=tk.LEFT)
        tk.Radiobutton(row,text="On", variable=self.debugState, value=True).pack(side="left")
        tk.Radiobutton(row,text="Off", variable=self.debugState, value=False).pack(side="left")

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        label = tk.Label(row, text = "Figure")
        label.pack(side=tk.TOP)
        f = tk.font.Font(label, label.cget("font"))
        f.configure(underline=True)
        label.configure(font=f)

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=25, text="Matplotlib color map", anchor='w')
        lab.pack(side=tk.LEFT)
        entry = tk.Entry(row,textvariable=self.figsettings["cmap"])
        entry.pack(side=tk.RIGHT, expand=tk.YES, fill=tk.X)
        button_tip(self.parent, entry, "Choose a matplotlib colormap. https://matplotlib.org/2.0.2/users/colormaps.html")


        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=25, text='Figure size [w",h"]', anchor='w')
        lab.pack(side=tk.LEFT)
        entry = tk.Entry(row,textvariable=self.figsettings["figsize"])
        entry.pack(side=tk.RIGHT, expand=tk.YES, fill=tk.X)

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=25, text="Font size", anchor='w')
        lab.pack(side=tk.LEFT)
        entry = tk.Entry(row,textvariable=self.figsettings["fontsize"])
        entry.pack(side=tk.RIGHT, expand=tk.YES, fill=tk.X)
        
        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=25, text="Labels:", anchor='w')
        lab.pack(side=tk.LEFT)

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=25, text="\tTitle:", anchor='w')
        lab.pack(side=tk.LEFT)
        tk.Radiobutton(row,text="On", variable=self.figsettings["figtitle"], value=True).pack(side="left")
        tk.Radiobutton(row,text="Off", variable=self.figsettings["figtitle"], value=False).pack(side="left")

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=25, text="\tX-axis:", anchor='w')
        lab.pack(side=tk.LEFT)
        tk.Radiobutton(row,text="On", variable=self.figsettings["figxaxis"], value=True).pack(side="left")
        tk.Radiobutton(row,text="Off", variable=self.figsettings["figxaxis"], value=False).pack(side="left")

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=25, text="\tY-axis:", anchor='w')
        lab.pack(side=tk.LEFT)
        tk.Radiobutton(row,text="On", variable=self.figsettings["figyaxis"], value=True).pack(side="left")
        tk.Radiobutton(row,text="Off", variable=self.figsettings["figyaxis"], value=False).pack(side="left")

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=25, text="Export figure vertical clip:", anchor='w')
        lab.pack(side=tk.LEFT)

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=25, text="\tTop:", anchor='w')
        lab.pack(side=tk.LEFT)
        entry = tk.Entry(row,textvariable=self.figsettings["figclip"][0])
        entry.pack(side=tk.RIGHT, expand=tk.YES, fill=tk.X)
        button_tip(self.parent, entry, "Top of figure vertical extent (Figure coordinates)")

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, fill=tk.X, padx=5, pady=5)
        lab = tk.Label(row, width=25, text="\tBottom:", anchor='w')
        lab.pack(side=tk.LEFT)
        entry = tk.Entry(row,textvariable=self.figsettings["figclip"][1])
        entry.pack(side=tk.RIGHT, expand=tk.YES, fill=tk.X)
        button_tip(self.parent, entry, "Bottom of figure vertical extent (Figure coordinates)")

        row = tk.Frame(settingsWindow)
        row.pack(side=tk.TOP, anchor="c")
        b = tk.Button(row, text="Save", command=lambda:[self.updateSettings(), settingsWindow.destroy()])
        b.pack(side="left")


    def check_cmap(self):
        # check if cmap is recognized
        try:
            plt.get_cmap(self.figsettings["cmap"].get())
        except:
            self.figsettings["cmap"].set("seismic")


    def updateSettings(self):
        try:
            float(self.epsEnt.get())
            self.eps_r.set(self.epsEnt.get())
        except:
            self.eps_r.set(3.15)

        # check if cmap is recognized
        self.check_cmap()

        # make sure fig size is of correct format
        size = self.figsettings["figsize"].get().split(",")
        if len(size) != 2:
            self.figsettings["figsize"].set("6.5,1.5")
        try:
            float(size[0])
            float(size[1])
        except:
            self.figsettings["figsize"].set("6.5,1.5")

        # make sure font and vertical clip are floats
        try:
            self.figsettings["fontsize"].get()
        except:
            self.figsettings["fontsize"].set(10)

        # make sure figure extents are between 0 and 1
        try:
            self.figsettings["figclip"][0].get()
            self.figsettings["figclip"][1].get()
            
            top = self.figsettings["figclip"][0].get()
            bottom = self.figsettings["figclip"][1].get()
            if top < 0:
                self.figsettings["figclip"][0].set(0)
            if bottom > 1:
                self.figsettings["figclip"][1].set(1)
            if top > bottom:
                self.figsettings["figclip"][0].set(0)
        except:
            self.figsettings["figclip"][0].set(0)
            self.figsettings["figclip"][1].set(1)

        # update figure settings
        self.impick.update_figsettings(self.figsettings)
        self.wvpick.update_figsettings(self.figsettings)

        # pass updated dielectric to impick
        self.impick.set_eps_r(self.eps_r.get())
        self.impick.set_axes()

        # pass updated debug state to impick
        self.impick.set_debugState(self.debugState.get())

        # draw images
        self.impick.drawData()


    def set_pick_vis(self):
        self.impick.show_picks(vis=self.pick_vis.get())


    def set_ann_vis(self):
        self.impick.show_labels(vis=self.ann_vis.get())


    def help(self):
        # help message box
        helpWindow = tk.Toplevel(self.parent)
        helpWindow.title("Instructions")
        helpWindow.config(bg="#d9d9d9")
        S = tk.Scrollbar(helpWindow)
        T = tk.Text(helpWindow, height=32, width=64, bg="#d9d9d9")
        S.pack(side=tk.RIGHT, fill=tk.Y)
        T.pack(side=tk.LEFT, fill=tk.Y)
        S.config(command=T.yview)
        T.config(yscrollcommand=S.set)
        note = """---Radar Analysis Graphical Utility---
                \n\n1. File->Open->Data File:    Load data file
                \n2. File->Open->Basemap File:    Load basemap
                \n3. Interpretation->New->Horizon:    Initialize new horizon
                \n4. Interpretation->Start:    Start picking
                \n5. [Spacebar]:    Toggle between radar and clutter images
                \n6. Click along reflector surface to pick horizon
                \n    [Backspace]:    Remove last pick
                \n7. Interpretation->Stop:    Stop picking
                \n8. File->Save->Picks:    Export picks
                \n9. File->Next:    Load next data file
                \n10. File->Exit:    Exit RAGU"""
        T.insert(tk.END, note)


    def shortcuts(self):
        # shortcut info
        shortcutWindow = tk.Toplevel(self.parent)
        shortcutWindow.title("Keyboard Shortcuts")
        shortcutWindow.config(bg="#d9d9d9")
        S = tk.Scrollbar(shortcutWindow)
        T = tk.Text(shortcutWindow, height=32, width=64, bg="#d9d9d9")
        S.pack(side=tk.RIGHT, fill=tk.Y)
        T.pack(side=tk.LEFT, fill=tk.Y)
        S.config(command=T.yview)
        T.config(yscrollcommand=S.set)
        note = """---General---
                \n\n[Ctrl+O]\t\t\tOpen data file
                \n[Ctrl+M]\t\t\tOpen basemap
                \n[Ctrl+S]\t\t\tSave picks
                \n[Ctrl+Q]\t\t\tExit RAGU
                \n\n---Profile View---
                \n\n[Spacebar]\t\t\tToggle between radar/clutter
                \n[H]\t\t\tReturn to home extent
                \n[A]\t\t\tPan left
                \n[D]\t\t\tPan right
                \n[W]\t\t\tPan up
                \n[S]\t\t\tPan down
                \n[Right]\t\t\tOpen next file in working directory
                \n\n---Picking---
                \n\n[Ctrl+N]\t\t\tStart picking / initialize new segment
                \n[Escape/Doubleclick]\t\t\tStop picking
                \n[Backspace]\t\t\tRemove last pick
                \n[C]\t\t\tRemove all interpretations
                \n\n---Waveform View---
                \n\n[H]\t\t\tReturn to home extent
                \n[Right]\t\t\tStep forward left
                \n[Left]\t\t\tStep backward"""
        T.insert(tk.END, note)


class popup():
    # initialize popup window
    def __init__(self, parent=None):
        self.parent = parent
        self.flag = 0     # 0 == closed safely, 1 == open, -1 == closed unsafely

    # new
    def new(self, title="title", bg="#d9d9d9", geom="500x150"):
        self.popup = tk.Toplevel(self.parent)
        self.popup.geometry(geom)
        self.popup.config(bg=bg)
        self.popup.title(title)
        self.popup.protocol("WM_DELETE_WINDOW", lambda flag=-1 : self.close(flag))
        self.flag = 1
        return self.popup

    # close
    def close(self, flag=0):
        self.popup.destroy()
        self.flag = flag


class button_tip(object):
    """
    popup info on gui button
    """
    def __init__(self, parent, widget, text="widget info"):
        self.waittime = 500     # miliseconds
        self.wraplength = 180   # pixels
        self.parent = parent    # parent frame
        self.widget = widget
        self.text = text
        self.widget.bind("<Enter>", self.enter)
        self.widget.bind("<Leave>", self.leave)
        self.widget.bind("<ButtonPress>", self.leave)
        self.id = None
        self.tw = None

    def enter(self, event=None):
        self.schedule()

    def leave(self, event=None):
        self.unschedule()
        self.hidetip()

    def schedule(self):
        self.unschedule()
        self.id = self.widget.after(self.waittime, self.showtip)

    def unschedule(self):
        id = self.id
        self.id = None
        if id:
            self.widget.after_cancel(id)

    def showtip(self, event=None):
        try:
            self.parent.update()    # update parent to get size
            bounds = (self.parent.winfo_rootx(),(self.parent.winfo_rootx()+self.parent.winfo_width()))
            x = y = 0
            x, y, cx, cy = self.widget.bbox("insert")
            x += self.widget.winfo_rootx() + 25
            y += self.widget.winfo_rooty() + 25

            # creates a toplevel window
            self.tw = tk.Toplevel(self.widget)
            # Leaves only the label and removes the app window
            self.tw.wm_overrideredirect(True)
            self.tw.wm_geometry("+%d+%d" % (x, y))
            label = tk.Label(self.tw, text=self.text, justify='left',
                        background="#ffffff", relief='solid', borderwidth=1,
                        wraplength = self.wraplength)
            label.pack(ipadx=1)
            self.tw.update()

            # move tip window left if outside parent window bounds
            if (x + self.tw.winfo_width()) > bounds[1]:
                self.tw.wm_geometry("+%d+%d" % (x-self.tw.winfo_width(), y))
                self.tw.update()
        except:
            pass

    def hidetip(self):
        tw = self.tw
        self.tw= None
        if tw:
            tw.destroy()

    
class project():
    # handle current project
    def __init__(self):
        self.projPath = ""
        self.datPath = ""
        self.mapPath = ""
        self.notePath = ""

    def set_projPath(self, path=""):
        self.projPath = path

    def get_projPath(self):
        return self.projPath

    def set_datPath(self, path=""):
        self.datPath = path

    def get_datPath(self):
        return self.datPath

    def set_mapPath(self, path=""):
        self.mapPath = path

    def get_mapPath(self):
        return self.mapPath

    def set_notePath(self, path=""):
        self.notePath = path

    def get_notePath(self):
        return self.notePath

    def update_paths(self, datPath="", mapPath="", notePath=""):
        if datPath:
            self.datPath = datPath
        if mapPath:
            self.mapPath = mapPath
        if notePath:
            self.notePath = notePath
    
    def save(self):
        if self.projPath:
            file = open(self.projPath,"w") 
            file.write("[paths]\n") 
            file.write("datPath = {}\n".format(self.datPath)) 
            file.write("mapPath = {}\n".format(self.mapPath)) 
            file.write("notePath = {}\n".format(self.notePath)) 
            file.close() 

    def load(self):
        if self.projPath:
            # read and parse config
            conf = configparser.ConfigParser()
            conf.read(self.projPath)
            self.set_datPath(conf["paths"]["datPath"])
            self.set_mapPath(conf["paths"]["mapPath"])
            self.set_notePath(conf["paths"]["notePath"])


"""
gps library contains classes and functions to handle different types of GPS data
modified from ImpDAR - DOI:10.5281/zenodo.3833057
"""
### imports ###
import numpy as np
from scipy.interpolate import interp1d
import matplotlib.pyplot as plt
import sys

class nmea_info:
    """Container for general information about lat, lon, etc.

    Attributes
    ----------
    lat: np.ndarray
        wgs84 latitude of points
    lon: np.ndarray_
        wgs84 longitude of points
    elev: np.ndarray
        elevation
    """

    all_data = None
    lat = None
    lon = None
    elev = None
    times = None
    scans = None

    def get_all(self):
        """Populate all the values from the input data."""
        self.glat()
        self.glon()
        self.ghgt()
        self.gtimes()

    def glat(self):
        """Populate lat(itude)."""
        if self.lat is None:
            self.lat = self.all_data[:, 2] * (
                (self.all_data[:, 1] - self.all_data[:, 1] % 100) / 100 + (
                    self.all_data[:, 1] % 100) / 60)
        return self.lat

    def glon(self):
        """Populate lon(gitude)."""
        if self.lon is None:
            self.lon = self.all_data[:, 4] * (
                (self.all_data[:, 3] - self.all_data[:, 3] % 100) / 100 + (
                    self.all_data[:, 3] % 100) / 60)
        return self.lon

    def ghgt(self):
        """Populate z (elevation)."""
        self.elev = self.all_data[:, 8]
        return self.elev

    def gtimes(self):
        """Populate times."""
        self.times = self.all_data[:, 0]
        return self.times


def nmea_all_info(list_of_sentences):
    """
    Return an object with the nmea info from a given list of sentences.

    Parameters
    ----------
    list_of_sentences : list of strs
        NMEA output.

    Raises
    ------
    ValueError
        If the NMEA output does not contain GGA strings.

    Returns
    -------
    np.ndarray
        An array of the useful information in the NMEA sentences.
    """
    def _gga_sentence_split(sentence):
        all = sentence.split(',')
        if len(all) > 5:
            numbers = list(map(lambda x: float(x) if x != '' else 0, all[1:3] + [1] + [all[4]] + [1] + all[6:10] + [all[11]]))
            if all[3] == 'S':
                numbers[2] = -1
            if all[5] == 'W':
                numbers[4] = -1
        elif len(all) > 2:
            numbers = list(map(lambda x: float(x) if x != '' else 0, all[1:3] + [1]))
            if all[3] == 'S':
                numbers[2] = -1
        else:
            numbers = np.nan
        return numbers

    if list_of_sentences[0].split(',')[0] == '$GPGGA':
        data = nmea_info()
        data.all_data = np.array([_gga_sentence_split(sentence)
                                  for sentence in list_of_sentences])
        return data
    else:
        print(list_of_sentences[0].split(',')[0])
        raise ValueError('I can only do gga sentences right now')


class GPSdat(nmea_info):
    """
    A container to make nmea info useful.

    This should handle frequency mismatch between radar and gps.

    Parameters
    ----------
    gga : list of strs
        The GPS data
    scans : list of floats
        traces in radargram for which gps data was acquired
    trace_num : np array
        array of traces in radar data over which to interpolate sparse gps data

    Returns
    -------
    None.

    """

    def __init__(self, gga, scans, trace_num):
        # parse recorded nmea strings
        self.nmea_info = nmea_all_info(gga)
        self.nmea_info.scans = scans
        self.nmea_info.get_all()
        # get time stamps where data recorded to interpolate between
        kgps_mask = np.logical_and(~np.isnan(self.nmea_info.times[1:]),
                                   np.diff(self.nmea_info.scans) != 0)
        kgps_mask = np.logical_and(np.diff(self.nmea_info.times) != 0,
                                   kgps_mask)
        kgps_where = np.where(kgps_mask)[0]
        kgps_indx = np.hstack((np.array([0]), 1 + kgps_where))
        # linearly interpolate lat,lon,elev,time - linearly extrapolate to fill tails 
        self.lat = interp1d(self.nmea_info.scans[kgps_indx],
                            self.nmea_info.lat[kgps_indx],
                            kind='linear',
                            fill_value='extrapolate')(np.arange(trace_num))
        self.lon = interp1d(self.nmea_info.scans[kgps_indx],
                            self.nmea_info.lon[kgps_indx],
                            kind='linear',
                            fill_value='extrapolate')(np.arange(trace_num))
        self.elev = interp1d(self.nmea_info.scans[kgps_indx],
                          self.nmea_info.elev[kgps_indx], kind='linear',
                          fill_value='extrapolate')(np.arange(trace_num))
        self.times = interp1d(self.nmea_info.scans[kgps_indx],
                              self.nmea_info.times[kgps_indx],
                              kind='linear',
                              fill_value='extrapolate')(np.arange(trace_num))


# RAGU - Radar Analysis Graphical Utility
#
# copyright © 2020 btobers <tobers.brandon@gmail.com>
#
# distributed under terms of the GNU GPL3.0 license
"""
wvpick class is a tkinter frame which handles the RAGU waveform view and radar pick optimization
"""
### imports ###
from ragu.tools import utils
import numpy as np
from scipy.interpolate import CubicSpline
from scipy.signal import find_peaks
import tkinter as tk
import sys,os,time,copy
import matplotlib as mpl
mpl.use("TkAgg")
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
# try:
#     plt.rcParams["font.family"] = "Times New Roman"
# except:
#     pass

class wvpick(tk.Frame):
    # wvpick is a class to optimize the picking of horizons from radar data
    def __init__(self, parent, button_tip, reset_picks, *args, **kwargs):
        tk.Frame.__init__(self, parent, *args, **kwargs)
        self.parent = parent
        self.button_tip = button_tip
        self.reset_picks = reset_picks
        self.winSize = tk.IntVar(value=100)
        self.stepSize = tk.IntVar(value=10)
        self.horVar = tk.StringVar()
        self.segVar = tk.IntVar()
        self.color = tk.StringVar()
        self.interp_type = tk.StringVar()
        self.interp_type.set("cubic")
        self.setup()

    def setup(self):
        # set up frames
        infoFrame0 = tk.Frame(self.parent)
        infoFrame0.pack(side="top",fill="both")        
        infoFrame = tk.Frame(infoFrame0)
        infoFrame.pack(side="left",fill="both")
        interpFrame = tk.Frame(infoFrame0, width=450)
        interpFrame.pack(side="right",fill="both")
        interpFrame.pack_propagate(0)
        toolbarFrame = tk.Frame(infoFrame)
        toolbarFrame.pack(side="bottom",fill="both")
        self.dataFrame = tk.Frame(self.parent)
        self.dataFrame.pack(side="bottom", fill="both", expand=1)

        # infoFrame exists for options to be added based on optimization needs
        tk.Label(infoFrame, text = "Amplitude Window [#Samples]: ").pack(side="left")
        entry = tk.Entry(infoFrame, textvariable=self.winSize, width = 5)
        entry.pack(side="left")
        self.button_tip(self.parent, entry, \
            "Pick amplitude window size. Window size (number of samples) centered on \
            manual picks from which to select maximum amplitude using auto-pick optimization.")
        tk.ttk.Separator(infoFrame,orient="vertical").pack(side="left", fill="both", padx=10, pady=4)

        tk.Label(infoFrame, text = "Step Size [#Traces]: ").pack(side="left")
        entry = tk.Entry(infoFrame, textvariable=self.stepSize, width = 5)
        entry.pack(side="left")
        self.button_tip(self.parent, entry, \
            "Number of traces to move forward and backwards in radargram upon ←/→ button or key press.")
        tk.ttk.Separator(infoFrame,orient="vertical").pack(side="left", fill="both", padx=10, pady=4)

        button = tk.Button(infoFrame, text="←", command = self.stepBackward, pady=0)
        button.pack(side="left")
        self.button_tip(self.parent, button, "Step backwards")
        button = tk.Button(infoFrame, text="→", command = self.stepForward, pady=0)
        button.pack(side="left")
        self.button_tip(self.parent, button, "Step forwards")
        tk.ttk.Separator(infoFrame,orient="vertical").pack(side="left", fill="both", padx=10, pady=4)

        # set up frame to hold pick information
        interpFrameT = tk.Frame(interpFrame)
        interpFrameT.pack(fill="both",expand=True)
        interpFrameT.pack_propagate(0)
        interpFrameB = tk.Frame(interpFrame)
        interpFrameB.pack(fill="both",expand=True)
        interpFrameB.pack_propagate(0)
        
        interpFrameTl = tk.Frame(interpFrameT,width=250,relief="ridge", borderwidth=1)
        interpFrameTl.pack(side="left",fill="both",expand=True)
        interpFrameTl.pack_propagate(0)
        interpFrameTr = tk.Frame(interpFrameT,width=200)
        interpFrameTr.pack(side="left",fill="both",expand=True)
        interpFrameTr.pack_propagate(0)

        interpFrameBl = tk.Frame(interpFrameB,width=250,relief="ridge", borderwidth=1)
        interpFrameBl.pack(side="left",fill="both",expand=True)
        interpFrameBl.pack_propagate(0)
        interpFrameBr = tk.Frame(interpFrameB,width=200)
        interpFrameBr.pack(side="left",fill="both",expand=True)
        interpFrameBr.pack_propagate(0)

        tk.Label(interpFrameTl,text="{0:<8}".format("Horizon:")).pack(side="left")
        button = tk.Button(interpFrameTl, text="Reset", width=6, command=self.reset)
        button.pack(side="right")
        self.button_tip(self.parent, button, "Reset pick data from profile interpretations")
        self.horMenu = tk.OptionMenu(interpFrameTl, self.horVar, *[None])
        self.horMenu.pack(side="right")
        self.horMenu.config(width=10)
        self.horVar.trace("w", self.update_seg_opt_menu) 
        self.horVar.trace("w", self.seg_select)
        self.horVar.trace("w", lambda *args, menu=self.horMenu : self.set_menu_color(menu))

        tk.Label(interpFrameBl,text="{0:<8}".format("Segment:")).pack(side="left")
        button = tk.Button(interpFrameBl, text=None, width=6, command=None)
        button.config(relief="sunken", state="disabled")
        button.pack(side="right")
        self.segMenu = tk.OptionMenu(interpFrameBl, self.segVar, *[None])
        self.segMenu.pack(side="right")
        self.segMenu.config(width=10)
        self.segVar.trace("w", self.first_trace)

        button = tk.Radiobutton(interpFrameTr, text="Linear", variable=self.interp_type, value="linear")
        button.pack(side="left",fill="both",expand=True)
        self.button_tip(self.parent, button, "Linear interpolation between manual pick modifications")

        button = tk.Radiobutton(interpFrameTr,text="Cubic Spline", variable=self.interp_type, value="cubic")
        button.pack(side="left",fill="both",expand=True)
        self.button_tip(self.parent, button, "Cubic spline interpolation between manual pick modifications")

        button = tk.Button(interpFrameBr, text="Auto-Optimize", command=self.auto_repick)
        button.pack(side="left",fill="both",expand=True)
        self.button_tip(self.parent, button, "Automatically optimize profile view interpretation picks for current horizon segment\
                                            by selecting maximum aplitude sample for each radargram trace within a window of the\
                                            specified size centered on existing profile picks.")

        button = tk.Button(interpFrameBr, text="Interpolate", command=self.interp_repick)
        button.pack(side="left",fill="both",expand=True)
        self.button_tip(self.parent, button, "Interpolate between manually updated waveform picks using the spefied interpolation method\
                                            for  current horizon segment.")

        # create figure object and datacanvas from it
        plt.rcParams.update({'font.size': 12})
        self.fig = mpl.figure.Figure()
        self.fig.patch.set_facecolor("#d9d9d9")
        self.dataCanvas = FigureCanvasTkAgg(self.fig, self.parent)
        self.dataCanvas.get_tk_widget().pack(in_=self.dataFrame, side="bottom", fill="both", expand=1) 
        self.click = self.fig.canvas.mpl_connect("button_press_event", self.onpress)
        self.unclick = self.fig.canvas.mpl_connect('button_release_event', self.onrelease)
        self.draw_cid = self.fig.canvas.mpl_connect("draw_event", self.update_bg)
        self.mousemotion = self.fig.canvas.mpl_connect('motion_notify_event', self.on_mouse_move)

        # add toolbar to plot
        self.toolbar = NavigationToolbar2Tk(self.dataCanvas, toolbarFrame)
        self.toolbar.pack(side="left")
        # self.toolbar.update()

        # create the figure axes
        self.ax = self.fig.add_subplot(111)
        self.fig.tight_layout(rect=[.02,.05,.97,1])
        self.ax.set_visible(False)

        # update the canvas
        self.dataCanvas._tkcanvas.pack()
        self.dataCanvas.draw()
    

    # set up variables
    def set_vars(self):
        self.rdata = None
        self.horizon_paths_opt = None
        self.ln_colors = {}
        self.horizons = []
        self.repick_idx = {}        # dictionary of indeces of repicked traces for each seg
        self.t  = None              # hold current trace number


    # set_data is a method to receive the radar data
    def set_data(self, rdata):
        # get data in dB
        self.rdata = rdata


    # receive horizon paths from impick
    def set_horizon_paths(self, horizon_paths):
        self.horizon_paths = copy.deepcopy(horizon_paths)
        self.horizon_paths_opt = copy.deepcopy(horizon_paths)
        self.horizons = list(self.horizon_paths_opt)
        self.nhorizons = len(self.horizons)
        self.update_hor_opt_menu()
        self.update_seg_opt_menu()

    
    # return optimized horizon paths
    def get_horizon_paths(self):
        return self.horizon_paths_opt


    # receive horizon line colors
    def set_horizon_colors(self, ln_colors):
        self.ln_colors = ln_colors


    # reset returns updated picks from gui
    def reset(self):
        if tk.messagebox.askyesno("Reset","Reset optimized horizon interpretations?"):
            self.reset_picks(force=True)


    # set_picks is a method which receives horizon interpretations for optimization
    def set_picks(self):
        self.ax.set_visible(True)
        # create lists of first and last picked trace number for each seg in each horizon
        self.segment_traces = {}
        # create dict to hold current trace number for each horizon
        self.trace = {}
        if self.nhorizons > 0:
            # iterate through horizon_paths
            for horizon, hdict in self.horizon_paths_opt.items():
                self.segment_traces[horizon] = bounds([],[])
                self.repick_idx[horizon] = []
                self.trace[horizon] = None
                # iterate through segments for each horizon
                for seg, path in hdict.items():
                    picked_traces = np.where(~np.isnan(path.x))[0]
                    if picked_traces.shape[0] > 0:
                        self.segment_traces[horizon].first.append(picked_traces[0])
                        self.segment_traces[horizon].last.append(picked_traces[-1])
                        if seg == 0:
                            self.trace[horizon] = picked_traces[0]
            # set horVar
            self.horVar.set(self.horizons[-1])
        else:
            self.trace[''] = 0


    # plot_wv is a method to draw the waveform on the datacanvas
    def plot_wv(self, *args):
        horizon = self.horVar.get()
        seg = self.segVar.get()
        winSize = self.winSize.get()
        # get previous x_lim
        xlim = self.ax.get_xlim()
        self.ax.clear()

        # plot trace power
        self.t = self.trace[horizon]
        self.ax.plot(self.rdata.proc.curr_dB[:,self.t], c="0.5")
        self.ax.set(xlabel = "Sample", ylabel = "Power [dB]", title="Trace: " + str(int(self.trace[horizon] + 1)) + "/" + str(int(self.rdata.tnum)))

        # get pick value range
        val = np.array(())

        # if self.nhorizons > 0:
        for horizon in self.horizons:
            # get sample index of pick for given trace
            for seg in self.horizon_paths_opt[horizon].keys():
                pick_idx0 = self.horizon_paths[horizon][seg].y[self.t]
                pick_idx1 = self.horizon_paths_opt[horizon][seg].y[self.t]
                val = np.append(val, (pick_idx0+pick_idx1)//2)
                if not np.isnan(pick_idx0):
                    self.ax.axvline(x = pick_idx0, c=self.ln_colors[horizon], label=horizon + "_" + str(seg))
                    if not np.isnan(pick_idx1) and (pick_idx0 != pick_idx1):
                        self.ax.axvline(x = pick_idx1, c=self.ln_colors[horizon], ls = "--", label=horizon + "_" + str(seg) + "_v2")
        
        self.ax.set(xlim=(0,self.rdata.snum))
        # save un-zoomed view to toolbar
        self.toolbar.push_current()

        # zoom in to window around horizons
        if not np.isnan(val).all():
            if xlim == (0,1) or xlim == (0,self.rdata.snum):
                min_ = np.nanmin(val) - (2*winSize)
                max_ = np.nanmax(val) + (2*winSize)
                self.ax.set(xlim=(min_,max_))
            else:
                self.ax.set(xlim=xlim)

        if len(self.ax.lines) > 1:
            self.ax.legend(loc="lower right")

        # initialize cursor crosshair lines
        self.horizontal_line = self.ax.axhline(color="r", lw=1, ls=":")
        self.vertical_line = self.ax.axvline(color="r", lw=1, ls=":")

        self.dataCanvas.draw()


    # full extent for trace
    def fullExtent(self):
        horizon = self.horVar.get()
        self.ax.set_xlim(0, self.rdata.snum)
        self.ax.set_ylim(self.rdata.proc.curr_dB[:,self.trace[horizon]].min(), self.rdata.proc.curr_dB[:,self.trace[horizon]].max())
        self.dataCanvas.draw()


    # stepBackward is a method to move backwards by the number of traces entered to stepSize
    def stepBackward(self):
        horizon = self.horVar.get()
        seg = self.segVar.get()
        step = self.stepSize.get()
        newTrace = self.trace[horizon] - step
        if self.nhorizons > 0:
            firstTrace_seg = self.segment_traces[horizon].first[seg]
            if newTrace >= firstTrace_seg:
                self.trace[horizon] -= step
            elif newTrace < firstTrace_seg:
                if self.trace[horizon] == firstTrace_seg:
                    return
                else:
                    self.trace[horizon] = firstTrace_seg

        else:
            if newTrace >= 0:
                self.trace[''] -= step
            elif newTrace < 0:
                if self.trace[''] == 0:
                    return
                else:
                    self.trace[''] = 0
            
        self.plot_wv()


    # stepForward is a method to move forward by the number of traces entered to stepSize
    def stepForward(self):
        horizon = self.horVar.get()
        seg = self.segVar.get()
        step = self.stepSize.get()
        newTrace = self.trace[horizon] + step
        if self.nhorizons > 0:
            lastTrace_seg = self.segment_traces[horizon].last[seg]
            if newTrace <= lastTrace_seg:
                self.trace[horizon] += step
            # if there are less traces left in the pick seg than the step size, move to the last trace in the seg
            elif newTrace > lastTrace_seg:
                if self.trace[horizon] == lastTrace_seg:
                    return
                    # if seg + 2 <= self.nhorizons and tk.messagebox.askokcancel("Next Sement","Finished optimization of current pick seg\n\tProceed to next seg?") == True:
                        # self.segVar.set(seg + 1) 
                else:
                    self.trace[horizon] = self.segment_traces[horizon].last[seg]
        
        else:
            if newTrace <= self.rdata.tnum:
                self.trace[''] += step
            elif newTrace > self.rdata.tnum:
                if self.trace[''] == self.rdata.tnum - 1:
                    return
                else:
                    self.trace[''] = self.rdata.tnum - 1

        self.plot_wv()


    # seg_select 
    def seg_select(self, *args):
        if self.t is None:
            return
        horizon = self.horVar.get()
        seg = self.segVar.get()
        self.trace[horizon] = self.t
        t0_list = self.segment_traces[horizon].first
        t1_list = self.segment_traces[horizon].last
        # if switched horizon, select segment which has current trace, if any
        for _i in range(len(t0_list)):  
            if (t0_list[_i] < self.t) and (t1_list[_i] > self.t):
                self.segVar.set(_i)
                self.t = None
                break
            else: 
                continue


    # first_trace updates the segment selection sets self.trace[horizon] to the first trace for the selected segment and replots
    def first_trace(self, *args):
        if self.t is not None:
            horizon = self.horVar.get()
            seg = self.segVar.get()
            t0_list = self.segment_traces[horizon].first
            if seg > len(t0_list):
                return
            self.trace[horizon] = t0_list[seg]
        # reset xlim
        self.ax.set(xlim=(0,self.rdata.snum))
        self.plot_wv()    


    # auto_repick is a method to automatically optimize subsurface picks by selecting the maximul amplitude sample within the specified window around existing picks
    def auto_repick(self):
        if self.nhorizons > 0:
            horizon = self.horVar.get()
            seg = self.segVar.get()
            winSize = self.winSize.get()
            x = np.arange(self.segment_traces[horizon].first[seg], self.segment_traces[horizon].last[seg] + 1)
            y = self.horizon_paths[horizon][seg].y[x]
            for _i in range(len(x)):
                if not np.isnan(y[_i]):
                    # find argmax for window for given data trace in pick
                    max_idx = np.nanargmax(self.rdata.proc.curr_dB[int(y[_i] - (winSize/2)):int(y[_i] + (winSize/2)), x[_i]])
                    # add argmax index to pick_dict1 - account for window index shift
                    self.horizon_paths_opt[horizon][seg].y[x[_i]] = max_idx + int(y[_i] - (winSize/2))
            self.plot_wv()


    # manual_repick is a method to manually adjust existing picks by clicking along the displayed waveform
    def manual_repick(self, event):
        if (not self.nhorizons > 0) or (event.inaxes != self.ax):
            return
        horizon = self.horVar.get()
        seg = self.segVar.get()
        # append trace number to repick_idx list to keep track of indeces for interpolation
        if (len(self.repick_idx[horizon]) == 0) or (self.repick_idx[horizon][-1] != self.trace[horizon]):
            self.repick_idx[horizon].append(self.trace[horizon])
        
        self.horizon_paths_opt[horizon][seg].y[self.trace[horizon]] = int(event.xdata)        
        self.plot_wv()


    # interp_repick is a method to interpolate between manually refined subsurface picks
    def interp_repick(self):
        horizon = self.horVar.get()
        seg = self.segVar.get()
        interp = self.interp_type.get()
        if len(self.repick_idx[horizon]) < 2:
            return
        # get indeces of repicked traces
        xp = self.repick_idx[horizon]
        # generate array of indices between first and last optimized pick
        interp_idx = np.arange(xp[0],xp[-1] + 1)

        if interp == "linear":
            # get twtt values at repicked indicesself.pick_dict1
            fp = self.horizon_paths_opt[horizon][seg].y[xp]
            # interpolate repicked values for seg
            self.horizon_paths_opt[horizon][seg].y[interp_idx] = np.interp(interp_idx, xp, fp)            

        elif interp == "cubic":
            # cubic spline between picks
            cs = CubicSpline(xp, self.horizon_paths_opt[horizon][seg].y[xp])
            # add cubic spline output interpolation to pick dictionary
            self.horizon_paths_opt[horizon][seg].y[interp_idx] = cs([interp_idx]).astype(int)


    # set tkinter menu font colors to match color name
    def set_menu_color(self, menu=None, *args):
        horizon = self.horVar.get()
        if not horizon:
            return
        c = self.ln_colors[horizon]
        menu.config(foreground=c, activeforeground=c, highlightcolor=c)


    # update the horizon menu
    def update_hor_opt_menu(self):
        self.horizons = list(self.horizon_paths_opt.keys())
        self.horMenu["menu"].delete(0, "end")
        for i, horizon in enumerate(self.horizons):
            c = self.ln_colors[horizon]
            self.horMenu["menu"].add_command(label=horizon, foreground=c, activeforeground=c, command=tk._setit(self.horVar, horizon))


    # update the horizon seg menu based on how many segments exist for given seg
    def update_seg_opt_menu(self, *args):
        horizon = self.horVar.get()
        self.segMenu["menu"].delete(0, "end")
        if horizon:
            for seg in sorted(self.horizon_paths_opt[horizon].keys()):
                self.segMenu["menu"].add_command(label=seg, command=tk._setit(self.segVar, seg))


    # show_artists
    def show_artists(self, val=True):
        for _i in self.ax.lines:
            _i.set_visible(val)


    # temporarily disconnect the draw_event callback to avoid recursion
    def safe_draw(self):
        canvas = self.fig.canvas
        canvas.mpl_disconnect(self.draw_cid)
        canvas.draw()
        self.draw_cid = canvas.mpl_connect("draw_event", self.update_bg)


    # when the figure is resized, hide picks, draw everything, and update the background image
    def update_bg(self, event=None):
        # temporarily hide artists
        self.show_artists(False)
        self.safe_draw()
        self.axbg = self.dataCanvas.copy_from_bbox(self.ax.bbox)
        # return artists visiblity to former state
        self.show_artists(True)
        self.blit()


    # update the figure, without needing to redraw the "axbg" artists
    def blit(self):
        self.fig.canvas.restore_region(self.axbg)
        for _i in self.ax.lines:
            self.ax.draw_artist(_i)
        self.fig.canvas.blit(self.ax.bbox)


    # onpress gets the time of the button_press_event
    def onpress(self,event):
        self.time_onclick = time.time()


    # onrelease calls manual_repick if the time between the button press and release events
    # is below a threshold so that segments aren't drawn while trying to zoom or pan
    def onrelease(self,event):
        if event.inaxes == self.ax:
            if event.button == 1 and ((time.time() - self.time_onclick) < 0.25):
                self.manual_repick(event)


    # on_mouse_move blit crosshairs
    def on_mouse_move(self, event):
        if self.rdata is None:
            return
        x = event.xdata
        y = event.ydata
        if event.inaxes == self.ax:
            y = self.rdata.proc.curr_dB[int(x),self.trace[self.horVar.get()]]
        self.horizontal_line.set_ydata(y)
        self.vertical_line.set_xdata(x)
        self.ax.figure.canvas.restore_region(self.axbg)
        self.ax.draw_artist(self.horizontal_line)
        self.ax.draw_artist(self.vertical_line)
        self.blit()

    # update_figsettings
    def update_figsettings(self, figsettings):
        self.figsettings = figsettings

        plt.rcParams.update({'font.size': self.figsettings["fontsize"].get()})

        for item in ([self.ax.title, self.ax.xaxis.label, self.ax.yaxis.label] +
                    self.ax.get_xticklabels() + self.ax.get_yticklabels()):
            item.set_fontsize(self.figsettings["fontsize"].get())

        self.ax.title.set_visible(self.figsettings["figtitle"].get())

        self.fig.canvas.draw()


    # clear is a method to clear the wavePick tab and stored data when a new track is loaded
    def clear(self):
        self.ax.clear()
        self.dataCanvas.draw()
        self.set_vars()


class bounds():
    # initialize a bounds object to hold first,last trace for interpretation segments - list or array like
    def __init__(self, first=None, last=None):
        self.first = first      # x array/list
        self.last = last      # y array/list